from __future__ import annotations
from enum import Enum
from typing import Union
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .plan_tiers import PlanTiers
from .plan_transform_usage import PlanTransformUsage
from .product import Product


class AggregateUsage(Enum):
    """An enumeration representing different categories.

    :cvar MAX: "max"
    :vartype MAX: str
    :cvar SUM: "sum"
    :vartype SUM: str
    :cvar LASTDURINGPERIOD: "last_during_period"
    :vartype LASTDURINGPERIOD: str
    :cvar LASTEVER: "last_ever"
    :vartype LASTEVER: str
    """

    MAX = "max"
    SUM = "sum"
    LASTDURINGPERIOD = "last_during_period"
    LASTEVER = "last_ever"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, AggregateUsage._member_map_.values()))


class BillingScheme(Enum):
    """An enumeration representing different categories.

    :cvar PERUNIT: "per_unit"
    :vartype PERUNIT: str
    :cvar TIERED: "tiered"
    :vartype TIERED: str
    """

    PERUNIT = "per_unit"
    TIERED = "tiered"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, BillingScheme._member_map_.values()))


class Interval(Enum):
    """An enumeration representing different categories.

    :cvar MONTH: "month"
    :vartype MONTH: str
    :cvar DAY: "day"
    :vartype DAY: str
    :cvar WEEK: "week"
    :vartype WEEK: str
    :cvar YEAR: "year"
    :vartype YEAR: str
    """

    MONTH = "month"
    DAY = "day"
    WEEK = "week"
    YEAR = "year"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Interval._member_map_.values()))


class PlanProductGuard(OneOfBaseModel):
    class_list = {"Product": Product, "str": str}


PlanProduct = Union[Product, str]


class TiersMode(Enum):
    """An enumeration representing different categories.

    :cvar GRADUATED: "graduated"
    :vartype GRADUATED: str
    :cvar VOLUME: "volume"
    :vartype VOLUME: str
    """

    GRADUATED = "graduated"
    VOLUME = "volume"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, TiersMode._member_map_.values()))


class UsageType(Enum):
    """An enumeration representing different categories.

    :cvar METERED: "metered"
    :vartype METERED: str
    :cvar LICENSED: "licensed"
    :vartype LICENSED: str
    """

    METERED = "metered"
    LICENSED = "licensed"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, UsageType._member_map_.values()))


@JsonMap({"id_": "id"})
class Plan(BaseModel):
    """Describes the pricing structure for the invoice item. For details of the fields in the `plan` object, see 'Create Plan'.

    :param active: Indicates whether the plan is available to be added to a subscription., defaults to None
    :type active: bool, optional
    :param aggregate_usage: Determines which quantity is used to calculate the pricing. One of the following:  * max - The maximum reported usage within the billing cycle.  * sum - The sum of all usage during a billing cycle. This is the default.  * last_during_period - The last usage reported within the billing cycle.  * last_ever - The last usage ever reported, if the latest billing cycles contain no usage at all. Required when usage_type is metered. Relevant when billing_scheme is set to per_unit. , defaults to None
    :type aggregate_usage: AggregateUsage, optional
    :param amount: The amount to charge, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal. For a free service, use 0. Must be null when tiers is set. Relevant when `billing_scheme` is set to **per_unit**., defaults to None
    :type amount: float, optional
    :param billing_scheme: Describes how to compute the price. One of the following:  * per_unit - The amount specified in amount is charged for each unit. Also set these related fields: amount, transform_usage, usage_type, aggregate_usage. This is the default.  * tiered - The unit pricing is computed using a tiering strategy as defined with the tiers and tiers_mode fields. , defaults to None
    :type billing_scheme: BillingScheme, optional
    :param created_at: Time the 'plan' object was created, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param id_: Unique ID for this plan. English alphanumeric characters with no special characters except underscore. If the client does not define an ID, Rapyd generates a string starting with **plan_**., defaults to None
    :type id_: str, optional
    :param interval: Specifies the units used in defining the billing cycle. One of the following:  * day  * week  * month  * year Maximum interval is 1 year.  , defaults to None
    :type interval: Interval, optional
    :param interval_count: Number of time intervals in the billing cycle. Integer, defaults to None
    :type interval_count: float, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param nickname: Brief description of the plan., defaults to None
    :type nickname: str, optional
    :param product: The ID of the product that this plan is for, and fields describing this product in the plan., defaults to None
    :type product: PlanProduct, optional
    :param tiers: Defines a tiered pricing structure. Each tier object represents a pricing tier., defaults to None
    :type tiers: List[PlanTiers], optional
    :param tiers_mode: Determines the mode for calculating the total tiered charge. One of the following values:  * graduated - The total cost at each price tier is calculated separately, then all tier charges are added together.  * volume - The total cost is calculated as the number of items times the applicable tier price. Relevant when billing_scheme is set to tiered , defaults to None
    :type tiers_mode: TiersMode, optional
    :param transform_usage: Defines the transformation that is applied to the reported usage before the billed price is computed. The transformation divides the quantity by the divisor specified in divide_by, then rounds up or down according to the setting in round. Relevant when billing_scheme is set to per_unit., defaults to None
    :type transform_usage: PlanTransformUsage, optional
    :param trial_period_days: Specifies the number of days before charges begin to accrue. Use this parameter to define a free trial period for a service., defaults to None
    :type trial_period_days: float, optional
    :param usage_type: Determines whether the customer is billed when the service is not actually used. Relevant when billing_scheme is set to per_unit. One of the following:  * metered - The customer is billed only for actual usage. You must also set aggregate_usage.  * licensed - The customer is billed even if the service is not used. This is the default. , defaults to None
    :type usage_type: UsageType, optional
    """

    def __init__(
        self,
        active: bool = None,
        aggregate_usage: AggregateUsage = None,
        amount: float = None,
        billing_scheme: BillingScheme = None,
        created_at: float = None,
        currency: str = None,
        id_: str = None,
        interval: Interval = None,
        interval_count: float = None,
        metadata: dict = None,
        nickname: str = None,
        product: PlanProduct = None,
        tiers: List[PlanTiers] = None,
        tiers_mode: TiersMode = None,
        transform_usage: PlanTransformUsage = None,
        trial_period_days: float = None,
        usage_type: UsageType = None,
    ):
        """Describes the pricing structure for the invoice item. For details of the fields in the `plan` object, see 'Create Plan'.

        :param active: Indicates whether the plan is available to be added to a subscription., defaults to None
        :type active: bool, optional
        :param aggregate_usage: Determines which quantity is used to calculate the pricing. One of the following:  * max - The maximum reported usage within the billing cycle.  * sum - The sum of all usage during a billing cycle. This is the default.  * last_during_period - The last usage reported within the billing cycle.  * last_ever - The last usage ever reported, if the latest billing cycles contain no usage at all. Required when usage_type is metered. Relevant when billing_scheme is set to per_unit. , defaults to None
        :type aggregate_usage: AggregateUsage, optional
        :param amount: The amount to charge, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal. For a free service, use 0. Must be null when tiers is set. Relevant when `billing_scheme` is set to **per_unit**., defaults to None
        :type amount: float, optional
        :param billing_scheme: Describes how to compute the price. One of the following:  * per_unit - The amount specified in amount is charged for each unit. Also set these related fields: amount, transform_usage, usage_type, aggregate_usage. This is the default.  * tiered - The unit pricing is computed using a tiering strategy as defined with the tiers and tiers_mode fields. , defaults to None
        :type billing_scheme: BillingScheme, optional
        :param created_at: Time the 'plan' object was created, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param id_: Unique ID for this plan. English alphanumeric characters with no special characters except underscore. If the client does not define an ID, Rapyd generates a string starting with **plan_**., defaults to None
        :type id_: str, optional
        :param interval: Specifies the units used in defining the billing cycle. One of the following:  * day  * week  * month  * year Maximum interval is 1 year.  , defaults to None
        :type interval: Interval, optional
        :param interval_count: Number of time intervals in the billing cycle. Integer, defaults to None
        :type interval_count: float, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param nickname: Brief description of the plan., defaults to None
        :type nickname: str, optional
        :param product: The ID of the product that this plan is for, and fields describing this product in the plan., defaults to None
        :type product: PlanProduct, optional
        :param tiers: Defines a tiered pricing structure. Each tier object represents a pricing tier., defaults to None
        :type tiers: List[PlanTiers], optional
        :param tiers_mode: Determines the mode for calculating the total tiered charge. One of the following values:  * graduated - The total cost at each price tier is calculated separately, then all tier charges are added together.  * volume - The total cost is calculated as the number of items times the applicable tier price. Relevant when billing_scheme is set to tiered , defaults to None
        :type tiers_mode: TiersMode, optional
        :param transform_usage: Defines the transformation that is applied to the reported usage before the billed price is computed. The transformation divides the quantity by the divisor specified in divide_by, then rounds up or down according to the setting in round. Relevant when billing_scheme is set to per_unit., defaults to None
        :type transform_usage: PlanTransformUsage, optional
        :param trial_period_days: Specifies the number of days before charges begin to accrue. Use this parameter to define a free trial period for a service., defaults to None
        :type trial_period_days: float, optional
        :param usage_type: Determines whether the customer is billed when the service is not actually used. Relevant when billing_scheme is set to per_unit. One of the following:  * metered - The customer is billed only for actual usage. You must also set aggregate_usage.  * licensed - The customer is billed even if the service is not used. This is the default. , defaults to None
        :type usage_type: UsageType, optional
        """
        self.active = active
        self.aggregate_usage = (
            self._enum_matching(
                aggregate_usage, AggregateUsage.list(), "aggregate_usage"
            )
            if aggregate_usage
            else None
        )
        self.amount = self._define_number("amount", amount, nullable=True)
        self.billing_scheme = (
            self._enum_matching(billing_scheme, BillingScheme.list(), "billing_scheme")
            if billing_scheme
            else None
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.interval = (
            self._enum_matching(interval, Interval.list(), "interval")
            if interval
            else None
        )
        self.interval_count = self._define_number(
            "interval_count", interval_count, nullable=True
        )
        self.metadata = metadata
        self.nickname = self._define_str("nickname", nickname, nullable=True)
        self.product = PlanProductGuard.return_one_of(product)
        self.tiers = self._define_list(tiers, PlanTiers)
        self.tiers_mode = (
            self._enum_matching(tiers_mode, TiersMode.list(), "tiers_mode")
            if tiers_mode
            else None
        )
        self.transform_usage = self._define_object(transform_usage, PlanTransformUsage)
        self.trial_period_days = self._define_number(
            "trial_period_days", trial_period_days, nullable=True
        )
        self.usage_type = (
            self._enum_matching(usage_type, UsageType.list(), "usage_type")
            if usage_type
            else None
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import InlineResponse200_40, V1AddressesBody


class PaymentAddressService(BaseService):

    @cast_models
    def create_address(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: V1AddressesBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_40:
        """Create an address.

        :param request_body: The request body., defaults to None
        :type request_body: V1AddressesBody, optional
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Creates an address.
        :rtype: InlineResponse200_40
        """

        Validator(V1AddressesBody).is_optional().validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/addresses", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_40._unmap(response)

    @cast_models
    def get_address(
        self,
        address_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_40:
        """Retrieve an address.

        :param address_id: ID of the address object. String starting with **address_**.
        :type address_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieves an address.
        :rtype: InlineResponse200_40
        """

        Validator(str).validate(address_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/addresses{{addressId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("addressId", address_id, explode=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_40._unmap(response)

    @cast_models
    def update_address(
        self,
        request_body: V1AddressesBody,
        address_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_40:
        """Change or modify an address. To clear a field, set it to an empty string.

        :param request_body: The request body.
        :type request_body: V1AddressesBody
        :param address_id: ID of the address object. String starting with **address_**.
        :type address_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Update an address.
        :rtype: InlineResponse200_40
        """

        Validator(V1AddressesBody).validate(request_body)
        Validator(str).validate(address_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/addresses{{addressId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("addressId", address_id, explode=True)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_40._unmap(response)

    @cast_models
    def delete_address(
        self,
        address_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_40:
        """Remove an address that is linked to a customer or is not linked to any object.

        :param address_id: ID of the address object. String starting with **address_**.
        :type address_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Removes an address.
        :rtype: InlineResponse200_40
        """

        Validator(str).validate(address_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/addresses{{addressId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("addressId", address_id, explode=True)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_40._unmap(response)



from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class AcceptSwift(Enum):
    """An enumeration representing different categories.

    :cvar TRUE: "true"
    :vartype TRUE: str
    :cvar FALSE: "false"
    :vartype FALSE: str
    """

    TRUE = "true"
    FALSE = "false"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, AcceptSwift._member_map_.values()))


class AccountIdType(Enum):
    """An enumeration representing different categories.

    :cvar IBANINTERNATIONALBANKACCOUNTNUMBERIBAN_: "iban - International bank account number (IBAN)."
    :vartype IBANINTERNATIONALBANKACCOUNTNUMBERIBAN_: str
    :cvar CLABECLABENUMBER_: "clabe - CLABE number."
    :vartype CLABECLABENUMBER_: str
    :cvar NULLREGULARBANKACCOUNTNUMBER_: "null - Regular bank account number."
    :vartype NULLREGULARBANKACCOUNTNUMBER_: str
    """

    IBANINTERNATIONALBANKACCOUNTNUMBERIBAN_ = (
        "iban - International bank account number (IBAN)."
    )
    CLABECLABENUMBER_ = "clabe - CLABE number."
    NULLREGULARBANKACCOUNTNUMBER_ = "null - Regular bank account number."

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, AccountIdType._member_map_.values()))


class LocalBankCodeType(Enum):
    """An enumeration representing different categories.

    :cvar BIC: "bic"
    :vartype BIC: str
    :cvar ABA: "aba"
    :vartype ABA: str
    :cvar SORTCODE: "sort_code"
    :vartype SORTCODE: str
    :cvar BSB: "bsb"
    :vartype BSB: str
    :cvar CNAPS: "cnaps"
    :vartype CNAPS: str
    :cvar IFSC: "ifsc"
    :vartype IFSC: str
    """

    BIC = "bic"
    ABA = "aba"
    SORTCODE = "sort_code"
    BSB = "bsb"
    CNAPS = "cnaps"
    IFSC = "ifsc"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, LocalBankCodeType._member_map_.values()))


class Refundable(Enum):
    """An enumeration representing different categories.

    :cvar TRUE: "true"
    :vartype TRUE: str
    :cvar FALSE: "false"
    :vartype FALSE: str
    """

    TRUE = "true"
    FALSE = "false"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Refundable._member_map_.values()))


class RemitterDetails(Enum):
    """An enumeration representing different categories.

    :cvar TRUE: "true"
    :vartype TRUE: str
    :cvar FALSE: "false"
    :vartype FALSE: str
    """

    TRUE = "true"
    FALSE = "false"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, RemitterDetails._member_map_.values()))


@JsonMap({})
class InlineResponse200_95Data(BaseModel):
    """InlineResponse200_95Data

    :param accept_swift: Indicates whether the virtual account has a SWIFT code., defaults to None
    :type accept_swift: AcceptSwift, optional
    :param account_id_type: Type of the virtual account number., defaults to None
    :type account_id_type: AccountIdType, optional
    :param country: Two-letter ISO 3166-1 ALPHA-2 code for the country. code., defaults to None
    :type country: str, optional
    :param local_bank_code_type: Type of the local bank code., defaults to None
    :type local_bank_code_type: LocalBankCodeType, optional
    :param refundable: Indicates whether the virtual account has a SWIFT code., defaults to None
    :type refundable: Refundable, optional
    :param remitter_details: Indicates whether remitter details are available., defaults to None
    :type remitter_details: RemitterDetails, optional
    :param supported_currencies: Array of currencies supported for the virtual account. Array of strings., defaults to None
    :type supported_currencies: List[str], optional
    """

    def __init__(
        self,
        accept_swift: AcceptSwift = None,
        account_id_type: AccountIdType = None,
        country: str = None,
        local_bank_code_type: LocalBankCodeType = None,
        refundable: Refundable = None,
        remitter_details: RemitterDetails = None,
        supported_currencies: List[str] = None,
    ):
        """InlineResponse200_95Data

        :param accept_swift: Indicates whether the virtual account has a SWIFT code., defaults to None
        :type accept_swift: AcceptSwift, optional
        :param account_id_type: Type of the virtual account number., defaults to None
        :type account_id_type: AccountIdType, optional
        :param country: Two-letter ISO 3166-1 ALPHA-2 code for the country. code., defaults to None
        :type country: str, optional
        :param local_bank_code_type: Type of the local bank code., defaults to None
        :type local_bank_code_type: LocalBankCodeType, optional
        :param refundable: Indicates whether the virtual account has a SWIFT code., defaults to None
        :type refundable: Refundable, optional
        :param remitter_details: Indicates whether remitter details are available., defaults to None
        :type remitter_details: RemitterDetails, optional
        :param supported_currencies: Array of currencies supported for the virtual account. Array of strings., defaults to None
        :type supported_currencies: List[str], optional
        """
        self.accept_swift = (
            self._enum_matching(accept_swift, AcceptSwift.list(), "accept_swift")
            if accept_swift
            else None
        )
        self.account_id_type = (
            self._enum_matching(
                account_id_type, AccountIdType.list(), "account_id_type"
            )
            if account_id_type
            else None
        )
        self.country = self._define_str("country", country, nullable=True)
        self.local_bank_code_type = (
            self._enum_matching(
                local_bank_code_type, LocalBankCodeType.list(), "local_bank_code_type"
            )
            if local_bank_code_type
            else None
        )
        self.refundable = (
            self._enum_matching(refundable, Refundable.list(), "refundable")
            if refundable
            else None
        )
        self.remitter_details = (
            self._enum_matching(
                remitter_details, RemitterDetails.list(), "remitter_details"
            )
            if remitter_details
            else None
        )
        self.supported_currencies = supported_currencies



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class V1PlansBody(BaseModel):
    """V1PlansBody

    :param aggregate_usage: Determines which quantity is used to calculate the pricing. Relevant when usage_type is metered. Default is sum., defaults to None
    :type aggregate_usage: str, optional
    :param amount: The amount to charge in the billing cycle. For a free service, use 0. Relevant when billing_scheme is set to per_unit. When the billing_scheme is set to tiered, set the amount in the tiers array., defaults to None
    :type amount: float, optional
    :param billing_scheme: Describes how to compute the price per billing period. One of the following values - per_unit, tiered, defaults to None
    :type billing_scheme: str, optional
    :param currency: Three-letter ISO 4217 code for the currency used in fields that represent monetary amounts. Uppercase.
    :type currency: str
    :param id_: Unique ID for this payment plan. English alphanumeric characters and underscore. Limited to 44 characters. If the merchant does not define an ID, Rapyd generates a string starting with plan_., defaults to None
    :type id_: str, optional
    :param interval: Specifies the units used in defining the billing cycle.
    :type interval: str
    :param interval_count: Number of intervals in the billing cycle. Default is 1., defaults to None
    :type interval_count: float, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param nickname: Brief description of the pricing plan., defaults to None
    :type nickname: str, optional
    :param product: ID of the 'product' object that this plan is for. The product must have type set to service.
    :type product: str
    :param tiers: Defines a tiered pricing structure. Array of objects. Must be null when billing_scheme is set to per_unit., defaults to None
    :type tiers: str, optional
    :param tiers_mode: Determines the mode for calculating the total tiered charge., defaults to None
    :type tiers_mode: str, optional
    :param transform_usage: Defines the transformation that is applied to the reported usage before the billed price is computed., defaults to None
    :type transform_usage: dict, optional
    :param trial_period_days: Specifies the number of days before charges begin to accrue. Use this parameter to define a free trial period for a service., defaults to None
    :type trial_period_days: float, optional
    :param usage_type: Determines whether the customer is billed when the service is not actually used. One of the following values - metered, licensed, defaults to None
    :type usage_type: str, optional
    """

    def __init__(
        self,
        currency: str,
        interval: str,
        product: str,
        aggregate_usage: str = None,
        amount: float = None,
        billing_scheme: str = None,
        id_: str = None,
        interval_count: float = None,
        metadata: dict = None,
        nickname: str = None,
        tiers: str = None,
        tiers_mode: str = None,
        transform_usage: dict = None,
        trial_period_days: float = None,
        usage_type: str = None,
    ):
        """V1PlansBody

        :param aggregate_usage: Determines which quantity is used to calculate the pricing. Relevant when usage_type is metered. Default is sum., defaults to None
        :type aggregate_usage: str, optional
        :param amount: The amount to charge in the billing cycle. For a free service, use 0. Relevant when billing_scheme is set to per_unit. When the billing_scheme is set to tiered, set the amount in the tiers array., defaults to None
        :type amount: float, optional
        :param billing_scheme: Describes how to compute the price per billing period. One of the following values - per_unit, tiered, defaults to None
        :type billing_scheme: str, optional
        :param currency: Three-letter ISO 4217 code for the currency used in fields that represent monetary amounts. Uppercase.
        :type currency: str
        :param id_: Unique ID for this payment plan. English alphanumeric characters and underscore. Limited to 44 characters. If the merchant does not define an ID, Rapyd generates a string starting with plan_., defaults to None
        :type id_: str, optional
        :param interval: Specifies the units used in defining the billing cycle.
        :type interval: str
        :param interval_count: Number of intervals in the billing cycle. Default is 1., defaults to None
        :type interval_count: float, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param nickname: Brief description of the pricing plan., defaults to None
        :type nickname: str, optional
        :param product: ID of the 'product' object that this plan is for. The product must have type set to service.
        :type product: str
        :param tiers: Defines a tiered pricing structure. Array of objects. Must be null when billing_scheme is set to per_unit., defaults to None
        :type tiers: str, optional
        :param tiers_mode: Determines the mode for calculating the total tiered charge., defaults to None
        :type tiers_mode: str, optional
        :param transform_usage: Defines the transformation that is applied to the reported usage before the billed price is computed., defaults to None
        :type transform_usage: dict, optional
        :param trial_period_days: Specifies the number of days before charges begin to accrue. Use this parameter to define a free trial period for a service., defaults to None
        :type trial_period_days: float, optional
        :param usage_type: Determines whether the customer is billed when the service is not actually used. One of the following values - metered, licensed, defaults to None
        :type usage_type: str, optional
        """
        self.aggregate_usage = self._define_str(
            "aggregate_usage", aggregate_usage, nullable=True
        )
        self.amount = self._define_number("amount", amount, nullable=True)
        self.billing_scheme = self._define_str(
            "billing_scheme", billing_scheme, nullable=True
        )
        self.currency = currency
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.interval = interval
        self.interval_count = self._define_number(
            "interval_count", interval_count, nullable=True
        )
        self.metadata = metadata
        self.nickname = self._define_str("nickname", nickname, nullable=True)
        self.product = product
        self.tiers = self._define_str("tiers", tiers, nullable=True)
        self.tiers_mode = self._define_str("tiers_mode", tiers_mode, nullable=True)
        self.transform_usage = transform_usage
        self.trial_period_days = self._define_number(
            "trial_period_days", trial_period_days, nullable=True
        )
        self.usage_type = self._define_str("usage_type", usage_type, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardsStatusBody(BaseModel):
    """CardsStatusBody

    :param blocked_reason: Reason for blocking the card. Relevant when the value of status is block., defaults to None
    :type blocked_reason: str, optional
    :param card: The card token, which is a string starting with **card_**. If the client is PCI-certified, you can use the actual card number.
    :type card: str
    :param status: Status of the card. One of the following, block/unblock
    :type status: str
    """

    def __init__(self, card: str, status: str, blocked_reason: str = None):
        """CardsStatusBody

        :param blocked_reason: Reason for blocking the card. Relevant when the value of status is block., defaults to None
        :type blocked_reason: str, optional
        :param card: The card token, which is a string starting with **card_**. If the client is PCI-certified, you can use the actual card number.
        :type card: str
        :param status: Status of the card. One of the following, block/unblock
        :type status: str
        """
        self.blocked_reason = self._define_str(
            "blocked_reason", blocked_reason, nullable=True
        )
        self.card = card
        self.status = status



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class IssuingBankaccountsBody(BaseModel):
    """IssuingBankaccountsBody

    :param country: The country that the account number will be associated with. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase.
    :type country: str
    :param currency: Currency of the account. Three-letter ISO 4217 code.
    :type currency: str
    :param description: Description of the account., defaults to None
    :type description: str, optional
    :param ewallet: ID of the wallet the bank account number is issued to. String starting with **ewallet_**.
    :type ewallet: str
    :param merchant_reference_id: ID of this account, as provided by the merchant. Limited to 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param requested_currency: Currency received by the virtual account after conversion. Uppercase. One of the following:\<BR\> * AUD - Australian Dollar\<BR\> * EUR - Euro\<BR\> * GBP - Pound Sterling\<BR\> * HKD - Hong Kong Dollar\<BR\> * SGD - Singapore Dollar\<BR\> * USD - US Dollar\<BR\> When not specified, the funds appear in the wallet’s currency account for the currency of the transaction., defaults to None
    :type requested_currency: str, optional
    """

    def __init__(
        self,
        country: str,
        currency: str,
        ewallet: str,
        description: str = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
        requested_currency: str = None,
    ):
        """IssuingBankaccountsBody

        :param country: The country that the account number will be associated with. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase.
        :type country: str
        :param currency: Currency of the account. Three-letter ISO 4217 code.
        :type currency: str
        :param description: Description of the account., defaults to None
        :type description: str, optional
        :param ewallet: ID of the wallet the bank account number is issued to. String starting with **ewallet_**.
        :type ewallet: str
        :param merchant_reference_id: ID of this account, as provided by the merchant. Limited to 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param requested_currency: Currency received by the virtual account after conversion. Uppercase. One of the following:\<BR\> * AUD - Australian Dollar\<BR\> * EUR - Euro\<BR\> * GBP - Pound Sterling\<BR\> * HKD - Hong Kong Dollar\<BR\> * SGD - Singapore Dollar\<BR\> * USD - US Dollar\<BR\> When not specified, the funds appear in the wallet’s currency account for the currency of the transaction., defaults to None
        :type requested_currency: str, optional
        """
        self.country = country
        self.currency = currency
        self.description = self._define_str("description", description, nullable=True)
        self.ewallet = ewallet
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.requested_currency = self._define_str(
            "requested_currency", requested_currency, nullable=True
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .verify_hosted_app_response_merchant_details import (
    VerifyHostedAppResponseMerchantDetails,
)


@JsonMap({})
class VerifyHostedAppResponse(BaseModel):
    """VerifyHostedAppResponse

    :param token: Identifier of the hosted application. String starting with happ_., defaults to None
    :type token: str, optional
    :param rapyd_entity_token: The ID of the Rapyd wallet of the company. String starting with ewallet_. Must be a company type wallet., defaults to None
    :type rapyd_entity_token: str, optional
    :param cancel_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param complete_url: URL where the customer is redirected after pressing Close to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param client_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
    :type client_reference_id: str, optional
    :param application_token: Identifier of the application. String starting with app_., defaults to None
    :type application_token: str, optional
    :param phone_number: The phone number of the applicant. This is the phone number where an authentication code is sent. Must have a leading plus sign (+)., defaults to None
    :type phone_number: str, optional
    :param merchant_details: Object containing information about the merchant., defaults to None
    :type merchant_details: VerifyHostedAppResponseMerchantDetails, optional
    :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
    :type redirect_url: str, optional
    :param metadata: A JSON object defined by the client, defaults to None
    :type metadata: dict, optional
    :param authorized_user_email: authorized_user_email, defaults to None
    :type authorized_user_email: str, optional
    """

    def __init__(
        self,
        token: str = None,
        rapyd_entity_token: str = None,
        cancel_url: str = None,
        complete_url: str = None,
        client_reference_id: str = None,
        application_token: str = None,
        phone_number: str = None,
        merchant_details: VerifyHostedAppResponseMerchantDetails = None,
        redirect_url: str = None,
        metadata: dict = None,
        authorized_user_email: str = None,
    ):
        """VerifyHostedAppResponse

        :param token: Identifier of the hosted application. String starting with happ_., defaults to None
        :type token: str, optional
        :param rapyd_entity_token: The ID of the Rapyd wallet of the company. String starting with ewallet_. Must be a company type wallet., defaults to None
        :type rapyd_entity_token: str, optional
        :param cancel_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param complete_url: URL where the customer is redirected after pressing Close to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param client_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
        :type client_reference_id: str, optional
        :param application_token: Identifier of the application. String starting with app_., defaults to None
        :type application_token: str, optional
        :param phone_number: The phone number of the applicant. This is the phone number where an authentication code is sent. Must have a leading plus sign (+)., defaults to None
        :type phone_number: str, optional
        :param merchant_details: Object containing information about the merchant., defaults to None
        :type merchant_details: VerifyHostedAppResponseMerchantDetails, optional
        :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
        :type redirect_url: str, optional
        :param metadata: A JSON object defined by the client, defaults to None
        :type metadata: dict, optional
        :param authorized_user_email: authorized_user_email, defaults to None
        :type authorized_user_email: str, optional
        """
        self.token = self._define_str("token", token, nullable=True)
        self.rapyd_entity_token = self._define_str(
            "rapyd_entity_token", rapyd_entity_token, nullable=True
        )
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.client_reference_id = self._define_str(
            "client_reference_id", client_reference_id, nullable=True
        )
        self.application_token = self._define_str(
            "application_token", application_token, nullable=True
        )
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.merchant_details = self._define_object(
            merchant_details, VerifyHostedAppResponseMerchantDetails
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.metadata = metadata
        self.authorized_user_email = self._define_str(
            "authorized_user_email", authorized_user_email, nullable=True
        )



from enum import Enum


class EntityTypeVerify(Enum):
    """An enumeration representing different categories.

    :cvar PARTNERSHIP: "Partnership"
    :vartype PARTNERSHIP: str
    :cvar SOLEPROPRIETOR: "Sole Proprietor"
    :vartype SOLEPROPRIETOR: str
    :cvar PRIVATELIMITEDCOMPANY: "Private Limited Company"
    :vartype PRIVATELIMITEDCOMPANY: str
    :cvar CHARITYORNPO: "Charity or NPO"
    :vartype CHARITYORNPO: str
    :cvar INDIVIDUAL: "Individual"
    :vartype INDIVIDUAL: str
    """

    PARTNERSHIP = "Partnership"
    SOLEPROPRIETOR = "Sole Proprietor"
    PRIVATELIMITEDCOMPANY = "Private Limited Company"
    CHARITYORNPO = "Charity or NPO"
    INDIVIDUAL = "Individual"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, EntityTypeVerify._member_map_.values()))



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class OrdersOrderIdBodyStatus(Enum):
    """An enumeration representing different categories.

    :cvar PAID: "paid"
    :vartype PAID: str
    :cvar PENDING: "pending"
    :vartype PENDING: str
    :cvar CANCELED: "canceled"
    :vartype CANCELED: str
    :cvar FULFILLED: "fulfilled"
    :vartype FULFILLED: str
    :cvar RETURNED: "returned"
    :vartype RETURNED: str
    :cvar PARTIAL: "partial"
    :vartype PARTIAL: str
    """

    PAID = "paid"
    PENDING = "pending"
    CANCELED = "canceled"
    FULFILLED = "fulfilled"
    RETURNED = "returned"
    PARTIAL = "partial"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, OrdersOrderIdBodyStatus._member_map_.values())
        )


@JsonMap({})
class OrdersOrderIdBody(BaseModel):
    """OrdersOrderIdBody

    :param coupon: ID of a coupon that is applied against this order. String starting with **coupon_**. The duration field of the coupon must be set to **repeating**, and the `duration_in_months` and `discount_duration_in_uses` fields must be set to **1**., defaults to None
    :type coupon: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param tax_percent: Percentage of tax to charge. Decimal., defaults to None
    :type tax_percent: float, optional
    :param status: Indicates the status of the order. Can be changed when status is pending, paid or fulfilled, defaults to None
    :type status: OrdersOrderIdBodyStatus, optional
    """

    def __init__(
        self,
        coupon: str = None,
        metadata: dict = None,
        tax_percent: float = None,
        status: OrdersOrderIdBodyStatus = None,
    ):
        """OrdersOrderIdBody

        :param coupon: ID of a coupon that is applied against this order. String starting with **coupon_**. The duration field of the coupon must be set to **repeating**, and the `duration_in_months` and `discount_duration_in_uses` fields must be set to **1**., defaults to None
        :type coupon: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param tax_percent: Percentage of tax to charge. Decimal., defaults to None
        :type tax_percent: float, optional
        :param status: Indicates the status of the order. Can be changed when status is pending, paid or fulfilled, defaults to None
        :type status: OrdersOrderIdBodyStatus, optional
        """
        self.coupon = self._define_str("coupon", coupon, nullable=True)
        self.metadata = metadata
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )
        self.status = (
            self._enum_matching(status, OrdersOrderIdBodyStatus.list(), "status")
            if status
            else None
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .entity_type import EntityType


@JsonMap({})
class PayoutsSenderBody(BaseModel):
    """PayoutsSenderBody

    :param company_name: Name of the sender company. Relevant when `entity_type` is **company**., defaults to None
    :type company_name: str, optional
    :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code.
    :type country: str
    :param currency: currency
    :type currency: str
    :param entity_type: entity_type
    :type entity_type: EntityType
    :param first_name: First name of the sebder. Relevant when entity_type is individual., defaults to None
    :type first_name: str, optional
    :param identification_type: Type of identification document for the sender., defaults to None
    :type identification_type: str, optional
    :param identification_value: Identification number on the document mentioned in identification_type., defaults to None
    :type identification_value: str, optional
    :param last_name: Last name of the sender. Relevant when `entity_type` is **individual**., defaults to None
    :type last_name: str, optional
    """

    def __init__(
        self,
        country: str,
        currency: str,
        entity_type: EntityType,
        company_name: str = None,
        first_name: str = None,
        identification_type: str = None,
        identification_value: str = None,
        last_name: str = None,
    ):
        """PayoutsSenderBody

        :param company_name: Name of the sender company. Relevant when `entity_type` is **company**., defaults to None
        :type company_name: str, optional
        :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code.
        :type country: str
        :param currency: currency
        :type currency: str
        :param entity_type: entity_type
        :type entity_type: EntityType
        :param first_name: First name of the sebder. Relevant when entity_type is individual., defaults to None
        :type first_name: str, optional
        :param identification_type: Type of identification document for the sender., defaults to None
        :type identification_type: str, optional
        :param identification_value: Identification number on the document mentioned in identification_type., defaults to None
        :type identification_value: str, optional
        :param last_name: Last name of the sender. Relevant when `entity_type` is **individual**., defaults to None
        :type last_name: str, optional
        """
        self.company_name = self._define_str(
            "company_name", company_name, nullable=True
        )
        self.country = country
        self.currency = self._define_str(
            "currency",
            currency,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.entity_type = self._enum_matching(
            entity_type, EntityType.list(), "entity_type"
        )
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.identification_type = self._define_str(
            "identification_type", identification_type, nullable=True
        )
        self.identification_value = self._define_str(
            "identification_value", identification_value, nullable=True
        )
        self.last_name = self._define_str("last_name", last_name, nullable=True)



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1_addresses_body import V1AddressesBody
from .contact_business import ContactBusiness


class ContactType(Enum):
    """An enumeration representing different categories.

    :cvar PERSONAL: "personal"
    :vartype PERSONAL: str
    :cvar BUSINESS: "business"
    :vartype BUSINESS: str
    """

    PERSONAL = "personal"
    BUSINESS = "business"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ContactType._member_map_.values()))


class ContactGender(Enum):
    """An enumeration representing different categories.

    :cvar MALE: "male"
    :vartype MALE: str
    :cvar FEMALE: "female"
    :vartype FEMALE: str
    :cvar OTHER: "other"
    :vartype OTHER: str
    :cvar NOTAPPLICABLE: "not_applicable"
    :vartype NOTAPPLICABLE: str
    """

    MALE = "male"
    FEMALE = "female"
    OTHER = "other"
    NOTAPPLICABLE = "not_applicable"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ContactGender._member_map_.values()))


class HouseType(Enum):
    """An enumeration representing different categories.

    :cvar LEASE: "lease"
    :vartype LEASE: str
    :cvar LIVEWITHFAMILY: "live_with_family"
    :vartype LIVEWITHFAMILY: str
    :cvar OWN: "own"
    :vartype OWN: str
    :cvar OWNER: "owner"
    :vartype OWNER: str
    :cvar MONTHTOMONTH: "month_to_month"
    :vartype MONTHTOMONTH: str
    :cvar HOUSINGPROJECT: "housing_project"
    :vartype HOUSINGPROJECT: str
    """

    LEASE = "lease"
    LIVEWITHFAMILY = "live_with_family"
    OWN = "own"
    OWNER = "owner"
    MONTHTOMONTH = "month_to_month"
    HOUSINGPROJECT = "housing_project"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, HouseType._member_map_.values()))


class MaritalStatus(Enum):
    """An enumeration representing different categories.

    :cvar MARRIED: "married"
    :vartype MARRIED: str
    :cvar SINGLE: "single"
    :vartype SINGLE: str
    :cvar DIVORCED: "divorced"
    :vartype DIVORCED: str
    :cvar WIDOWED: "widowed"
    :vartype WIDOWED: str
    :cvar COHABITING: "cohabiting"
    :vartype COHABITING: str
    :cvar NOTAPPLICABLE: "not_applicable"
    :vartype NOTAPPLICABLE: str
    """

    MARRIED = "married"
    SINGLE = "single"
    DIVORCED = "divorced"
    WIDOWED = "widowed"
    COHABITING = "cohabiting"
    NOTAPPLICABLE = "not_applicable"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, MaritalStatus._member_map_.values()))


class VerificationStatus(Enum):
    """An enumeration representing different categories.

    :cvar NOTVERIFIED: "not verified"
    :vartype NOTVERIFIED: str
    :cvar KYCD: "KYCd"
    :vartype KYCD: str
    """

    NOTVERIFIED = "not verified"
    KYCD = "KYCd"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, VerificationStatus._member_map_.values()))


@JsonMap({"id_": "id"})
class Contact(BaseModel):
    """Contact

    :param address: address associated with this specific Rapyd entity Payment/Customer etc..., defaults to None
    :type address: V1AddressesBody, optional
    :param business_details: business_details, defaults to None
    :type business_details: ContactBusiness, optional
    :param compliance_profile: Indicates the degree to which this contact can use the wallet., defaults to None
    :type compliance_profile: float, optional
    :param contact_type: Type of contact., defaults to None
    :type contact_type: ContactType, optional
    :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type country: str, optional
    :param created_at: created_at, defaults to None
    :type created_at: float, optional
    :param date_of_birth: Date of birth of the individua, defaults to None
    :type date_of_birth: str, optional
    :param email: Email address of the contact, defaults to None
    :type email: str, optional
    :param ewallet: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**., defaults to None
    :type ewallet: str, optional
    :param first_name: First name of the personal contact or primary person associated with the business contact., defaults to None
    :type first_name: str, optional
    :param gender: Gender of the personal contact or primary person associated with the business contact, defaults to None
    :type gender: ContactGender, optional
    :param house_type: Description of the type of residency at the contact's residence., defaults to None
    :type house_type: HouseType, optional
    :param id_: ID of the contact object. String starting with cont_., defaults to None
    :type id_: str, optional
    :param identification_number: ID number as shown by the ID document., defaults to None
    :type identification_number: str, optional
    :param identification_type: Type of the identification document associated with the contact. Uppercase., defaults to None
    :type identification_type: str, optional
    :param issued_card_data: Describes additional information about the business contact for a company wallet., defaults to None
    :type issued_card_data: dict, optional
    :param last_name: Family name of the personal contact or primary person associated with the business contact. Required for issuing a card to a personal contact., defaults to None
    :type last_name: str, optional
    :param marital_status: Marital status of the personal contact or primary person associated with the business contact., defaults to None
    :type marital_status: MaritalStatus, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param middle_name: Middle name of the personal contact or primary person associated with the business contact., defaults to None
    :type middle_name: str, optional
    :param mothers_name: Name of the contact's mother, defaults to None
    :type mothers_name: str, optional
    :param nationality: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type nationality: str, optional
    :param phone_number: Phone number of the contact in E.164 format., defaults to None
    :type phone_number: str, optional
    :param second_last_name: Second last name of the personal contact or primary person associated with the business contact., defaults to None
    :type second_last_name: str, optional
    :param send_notifications: Determines whether Rapyd sends notifications to the contact. Default is false., defaults to None
    :type send_notifications: bool, optional
    :param verification_status: Result of the verification check., defaults to None
    :type verification_status: VerificationStatus, optional
    :param contact_reference_id: Reference ID for the contact. Limited to 45 characters. Should be unique per merchant per e-wallet., defaults to None
    :type contact_reference_id: str, optional
    """

    def __init__(
        self,
        address: V1AddressesBody = None,
        business_details: ContactBusiness = None,
        compliance_profile: float = None,
        contact_type: ContactType = None,
        country: str = None,
        created_at: float = None,
        date_of_birth: str = None,
        email: str = None,
        ewallet: str = None,
        first_name: str = None,
        gender: ContactGender = None,
        house_type: HouseType = None,
        id_: str = None,
        identification_number: str = None,
        identification_type: str = None,
        issued_card_data: dict = None,
        last_name: str = None,
        marital_status: MaritalStatus = None,
        metadata: dict = None,
        middle_name: str = None,
        mothers_name: str = None,
        nationality: str = None,
        phone_number: str = None,
        second_last_name: str = None,
        send_notifications: bool = None,
        verification_status: VerificationStatus = None,
        contact_reference_id: str = None,
    ):
        """Contact

        :param address: address associated with this specific Rapyd entity Payment/Customer etc..., defaults to None
        :type address: V1AddressesBody, optional
        :param business_details: business_details, defaults to None
        :type business_details: ContactBusiness, optional
        :param compliance_profile: Indicates the degree to which this contact can use the wallet., defaults to None
        :type compliance_profile: float, optional
        :param contact_type: Type of contact., defaults to None
        :type contact_type: ContactType, optional
        :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type country: str, optional
        :param created_at: created_at, defaults to None
        :type created_at: float, optional
        :param date_of_birth: Date of birth of the individua, defaults to None
        :type date_of_birth: str, optional
        :param email: Email address of the contact, defaults to None
        :type email: str, optional
        :param ewallet: ID of the Rapyd Wallet that this contact is associated with. String starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        :param first_name: First name of the personal contact or primary person associated with the business contact., defaults to None
        :type first_name: str, optional
        :param gender: Gender of the personal contact or primary person associated with the business contact, defaults to None
        :type gender: ContactGender, optional
        :param house_type: Description of the type of residency at the contact's residence., defaults to None
        :type house_type: HouseType, optional
        :param id_: ID of the contact object. String starting with cont_., defaults to None
        :type id_: str, optional
        :param identification_number: ID number as shown by the ID document., defaults to None
        :type identification_number: str, optional
        :param identification_type: Type of the identification document associated with the contact. Uppercase., defaults to None
        :type identification_type: str, optional
        :param issued_card_data: Describes additional information about the business contact for a company wallet., defaults to None
        :type issued_card_data: dict, optional
        :param last_name: Family name of the personal contact or primary person associated with the business contact. Required for issuing a card to a personal contact., defaults to None
        :type last_name: str, optional
        :param marital_status: Marital status of the personal contact or primary person associated with the business contact., defaults to None
        :type marital_status: MaritalStatus, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param middle_name: Middle name of the personal contact or primary person associated with the business contact., defaults to None
        :type middle_name: str, optional
        :param mothers_name: Name of the contact's mother, defaults to None
        :type mothers_name: str, optional
        :param nationality: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type nationality: str, optional
        :param phone_number: Phone number of the contact in E.164 format., defaults to None
        :type phone_number: str, optional
        :param second_last_name: Second last name of the personal contact or primary person associated with the business contact., defaults to None
        :type second_last_name: str, optional
        :param send_notifications: Determines whether Rapyd sends notifications to the contact. Default is false., defaults to None
        :type send_notifications: bool, optional
        :param verification_status: Result of the verification check., defaults to None
        :type verification_status: VerificationStatus, optional
        :param contact_reference_id: Reference ID for the contact. Limited to 45 characters. Should be unique per merchant per e-wallet., defaults to None
        :type contact_reference_id: str, optional
        """
        self.address = self._define_object(address, V1AddressesBody)
        self.business_details = self._define_object(business_details, ContactBusiness)
        self.compliance_profile = self._define_number(
            "compliance_profile", compliance_profile, nullable=True
        )
        self.contact_type = (
            self._enum_matching(contact_type, ContactType.list(), "contact_type")
            if contact_type
            else None
        )
        self.country = self._define_str("country", country, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.date_of_birth = self._define_str(
            "date_of_birth", date_of_birth, nullable=True
        )
        self.email = self._define_str(
            "email",
            email,
            nullable=True,
            pattern="^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$",
        )
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.gender = (
            self._enum_matching(gender, ContactGender.list(), "gender")
            if gender
            else None
        )
        self.house_type = (
            self._enum_matching(house_type, HouseType.list(), "house_type")
            if house_type
            else None
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.identification_number = self._define_str(
            "identification_number", identification_number, nullable=True
        )
        self.identification_type = self._define_str(
            "identification_type", identification_type, nullable=True
        )
        self.issued_card_data = issued_card_data
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.marital_status = (
            self._enum_matching(marital_status, MaritalStatus.list(), "marital_status")
            if marital_status
            else None
        )
        self.metadata = metadata
        self.middle_name = self._define_str("middle_name", middle_name, nullable=True)
        self.mothers_name = self._define_str(
            "mothers_name", mothers_name, nullable=True
        )
        self.nationality = self._define_str("nationality", nationality, nullable=True)
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.second_last_name = self._define_str(
            "second_last_name", second_last_name, nullable=True
        )
        self.send_notifications = send_notifications
        self.verification_status = (
            self._enum_matching(
                verification_status, VerificationStatus.list(), "verification_status"
            )
            if verification_status
            else None
        )
        self.contact_reference_id = self._define_str(
            "contact_reference_id", contact_reference_id, nullable=True
        )



from enum import Enum


class PaymentStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACT: "ACT"
    :vartype ACT: str
    :cvar CLO: "CLO"
    :vartype CLO: str
    :cvar ERR: "ERR"
    :vartype ERR: str
    :cvar CAN: "CAN"
    :vartype CAN: str
    :cvar EXP: "EXP"
    :vartype EXP: str
    :cvar DLD: "DLD"
    :vartype DLD: str
    :cvar REV: "REV"
    :vartype REV: str
    :cvar UND: "UND"
    :vartype UND: str
    :cvar CRE: "CRE"
    :vartype CRE: str
    """

    ACT = "ACT"
    CLO = "CLO"
    ERR = "ERR"
    CAN = "CAN"
    EXP = "EXP"
    DLD = "DLD"
    REV = "REV"
    UND = "UND"
    CRE = "CRE"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, PaymentStatus._member_map_.values()))



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class InlineResponse200_111Data(BaseModel):
    """InlineResponse200_111Data

    :param id_: ID Verification., defaults to None
    :type id_: str, optional
    :param reference_id: Organization Reference Token., defaults to None
    :type reference_id: str, optional
    """

    def __init__(self, id_: str = None, reference_id: str = None):
        """InlineResponse200_111Data

        :param id_: ID Verification., defaults to None
        :type id_: str, optional
        :param reference_id: Organization Reference Token., defaults to None
        :type reference_id: str, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.reference_id = self._define_str(
            "reference_id", reference_id, nullable=True
        )



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .limit import Limit


@JsonMap({"id_": "id"})
class Account(BaseModel):
    """Account

    :param alias: Three-letter ISO 4217 code for the currency of the account., defaults to None
    :type alias: str, optional
    :param balance: Available funds in the account., defaults to None
    :type balance: float, optional
    :param currency: currency, defaults to None
    :type currency: str, optional
    :param id_: ID of the account. UUID, defaults to None
    :type id_: str, optional
    :param limit: limit, defaults to None
    :type limit: str, optional
    :param limits: limits, defaults to None
    :type limits: List[Limit], optional
    :param on_hold_balance: Amount in the on-hold balance of the account., defaults to None
    :type on_hold_balance: float, optional
    :param received_balance: Amount of escrow funds in the account., defaults to None
    :type received_balance: float, optional
    :param reserve_balance: Amount in the reserve balance of the account., defaults to None
    :type reserve_balance: float, optional
    """

    def __init__(
        self,
        alias: str = None,
        balance: float = None,
        currency: str = None,
        id_: str = None,
        limit: str = None,
        limits: List[Limit] = None,
        on_hold_balance: float = None,
        received_balance: float = None,
        reserve_balance: float = None,
    ):
        """Account

        :param alias: Three-letter ISO 4217 code for the currency of the account., defaults to None
        :type alias: str, optional
        :param balance: Available funds in the account., defaults to None
        :type balance: float, optional
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param id_: ID of the account. UUID, defaults to None
        :type id_: str, optional
        :param limit: limit, defaults to None
        :type limit: str, optional
        :param limits: limits, defaults to None
        :type limits: List[Limit], optional
        :param on_hold_balance: Amount in the on-hold balance of the account., defaults to None
        :type on_hold_balance: float, optional
        :param received_balance: Amount of escrow funds in the account., defaults to None
        :type received_balance: float, optional
        :param reserve_balance: Amount in the reserve balance of the account., defaults to None
        :type reserve_balance: float, optional
        """
        self.alias = self._define_str("alias", alias, nullable=True)
        self.balance = self._define_number("balance", balance, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.limit = self._define_str("limit", limit, nullable=True)
        self.limits = self._define_list(limits, Limit)
        self.on_hold_balance = self._define_number(
            "on_hold_balance", on_hold_balance, nullable=True
        )
        self.received_balance = self._define_number(
            "received_balance", received_balance, nullable=True
        )
        self.reserve_balance = self._define_number(
            "reserve_balance", reserve_balance, nullable=True
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .escrow_response import EscrowResponse


@JsonMap({})
class InlineResponse200_29(BaseModel):
    """InlineResponse200_29

    :param data: data, defaults to None
    :type data: EscrowResponse, optional
    """

    def __init__(self, data: EscrowResponse = None):
        """InlineResponse200_29

        :param data: data, defaults to None
        :type data: EscrowResponse, optional
        """
        self.data = self._define_object(data, EscrowResponse)



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .field_1_conditions import Field1Conditions


class Field1Type(Enum):
    """An enumeration representing different categories.

    :cvar BOOLEAN: "boolean"
    :vartype BOOLEAN: str
    :cvar NUMBER: "number"
    :vartype NUMBER: str
    :cvar STRING: "string"
    :vartype STRING: str
    """

    BOOLEAN = "boolean"
    NUMBER = "number"
    STRING = "string"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Field1Type._member_map_.values()))


@JsonMap({"type_": "type"})
class Field1(BaseModel):
    """Field Object

    :param code: Three-letter alphabetic ISO 4217 code for the official currency used in the country., defaults to None
    :type code: str, optional
    :param is_required: Whether the field is always required for using the payment method, defaults to None
    :type is_required: bool, optional
    :param is_updatable: Indicates whether the field can be updated with Update Payment Method, defaults to None
    :type is_updatable: bool, optional
    :param instructions: instructions, defaults to None
    :type instructions: str, optional
    :param name: Name of the currency in English., defaults to None
    :type name: str, optional
    :param numeric_code: Three-letter ISO numeric 4217 code for the currency., defaults to None
    :type numeric_code: str, optional
    :param required_fields: required_fields, defaults to None
    :type required_fields: List[Field1], optional
    :param type_: type_, defaults to None
    :type type_: Field1Type, optional
    :param regex: A regular expression that defines the format when type is string., defaults to None
    :type regex: str, optional
    :param conditions: Defines specific conditions when a field is required for a payment method. When the conditions defined by conditions are met, the field is required even though the value of is_required is false., defaults to None
    :type conditions: List[Field1Conditions], optional
    :param description: Description of the field., defaults to None
    :type description: str, optional
    """

    def __init__(
        self,
        code: str = None,
        is_required: bool = None,
        is_updatable: bool = None,
        instructions: str = None,
        name: str = None,
        numeric_code: str = None,
        required_fields: List[Field1] = None,
        type_: Field1Type = None,
        regex: str = None,
        conditions: List[Field1Conditions] = None,
        description: str = None,
    ):
        """Field Object

        :param code: Three-letter alphabetic ISO 4217 code for the official currency used in the country., defaults to None
        :type code: str, optional
        :param is_required: Whether the field is always required for using the payment method, defaults to None
        :type is_required: bool, optional
        :param is_updatable: Indicates whether the field can be updated with Update Payment Method, defaults to None
        :type is_updatable: bool, optional
        :param instructions: instructions, defaults to None
        :type instructions: str, optional
        :param name: Name of the currency in English., defaults to None
        :type name: str, optional
        :param numeric_code: Three-letter ISO numeric 4217 code for the currency., defaults to None
        :type numeric_code: str, optional
        :param required_fields: required_fields, defaults to None
        :type required_fields: List[Field1], optional
        :param type_: type_, defaults to None
        :type type_: Field1Type, optional
        :param regex: A regular expression that defines the format when type is string., defaults to None
        :type regex: str, optional
        :param conditions: Defines specific conditions when a field is required for a payment method. When the conditions defined by conditions are met, the field is required even though the value of is_required is false., defaults to None
        :type conditions: List[Field1Conditions], optional
        :param description: Description of the field., defaults to None
        :type description: str, optional
        """
        self.code = self._define_str("code", code, nullable=True)
        self.is_required = is_required
        self.is_updatable = is_updatable
        self.instructions = self._define_str(
            "instructions", instructions, nullable=True
        )
        self.name = self._define_str("name", name, nullable=True)
        self.numeric_code = self._define_str(
            "numeric_code", numeric_code, nullable=True
        )
        self.required_fields = self._define_list(required_fields, Field1)
        self.type_ = (
            self._enum_matching(type_, Field1Type.list(), "type_") if type_ else None
        )
        self.regex = self._define_str("regex", regex, nullable=True)
        self.conditions = self._define_list(conditions, Field1Conditions)
        self.description = self._define_str("description", description, nullable=True)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class Duration(Enum):
    """An enumeration representing different categories.

    :cvar FOREVER: "forever"
    :vartype FOREVER: str
    :cvar REPEATING: "repeating"
    :vartype REPEATING: str
    """

    FOREVER = "forever"
    REPEATING = "repeating"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Duration._member_map_.values()))


@JsonMap({"id_": "id"})
class Coupon(BaseModel):
    """The coupon can be applied to the following objects - * Subscription - All charges in the subscription are subject to the discount described in the coupon. To add a coupon to a subscription, first use Create Coupon. Then add the coupon ID to the coupon field of the subscription with Create Subscription or Update Subscription. * Customer - The discount applies to all charges to the customer. To add a coupon to a customer, first use Create Coupon. Then add the coupon ID to the coupon field of the customer with Create Customer or Update Customer. * Order - The discount applies to a single order. To add a coupon to an order, first use Create Coupon. The duration field of the coupon must be set to repeating, and the duration_in_months and discount_duration_in_uses fields must be set to 1. Then add the coupon ID to the coupon field of the order with Create an Order or Update Order. A coupon can be applied to one or more customers or subscriptions. However, a subscription can have only one coupon, and the only way to assign two or more coupons to a customer is to assign them to separate subscriptions. If there is a coupon for the customer and another coupon for the customer's subscription, the subscription coupon takes precedence.

    :param amount_off: The amount of money to subtract from the payment. Decimal. Range: Positive decimal number, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
    :type amount_off: float, optional
    :param created: When the coupon was created, in Unix time. Response only., defaults to None
    :type created: float, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param description: Description of the coupon., defaults to None
    :type description: str, optional
    :param discount_duration_in_uses: The number of times that the coupon can be redeemed by a specific customer or subscription. Relevant when duration is repeating and redeem_by is not set., defaults to None
    :type discount_duration_in_uses: float, optional
    :param discount_valid_until: The final date that a discount can be used, in Unix time. Relevant when discount_validity_in_months is not set., defaults to None
    :type discount_valid_until: float, optional
    :param discount_validity_in_months: The number of months after the discount is created that it can be used. Relevant when discount_valid_until is not set., defaults to None
    :type discount_validity_in_months: float, optional
    :param duration: Determines how long the discount remains in effect. One of the following:  * forever  * repeating , defaults to None
    :type duration: Duration, optional
    :param duration_in_months: Determines the number of months that the coupon remains in effect after its creation. Integer. Required when duration is repeating., defaults to None
    :type duration_in_months: float, optional
    :param id_: Unique ID for this coupon. English alphanumeric characters with no special characters except underscore. If the merchant does not define an ID, Rapyd generates a string starting with **coupon_**., defaults to None
    :type id_: str, optional
    :param max_redemptions: Determines the number of times the coupon can be redeemed. The number of customers, subscriptions or orders that the coupon is applied to cannot exceed this number. Integer., defaults to None
    :type max_redemptions: float, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param percent_off: Determines the size of the discount, measured in percent. Integer. Range: 1-100., defaults to None
    :type percent_off: float, optional
    :param redeem_by: The last time at which the coupon can be redeemed, in Unix time. After the redeem_by date, the coupon can no longer be applied to new customers. 'redeem_by' takes precedence over the setting of **discount_duration_in_uses** and **duration_in_months**., defaults to None
    :type redeem_by: float, optional
    :param times_redeemed: Indicates the number of times the coupon has been applied to a customer, subscription or invoice. If the discount has been used more than one time by a customer, subscription or invoice, the number of redemptions after the first is also added to this value. Integer. When this value equals **max_redemptions**, the coupon becomes no longer valid. Response only., defaults to None
    :type times_redeemed: float, optional
    :param valid: Indicates whether the coupon can be redeemed., defaults to None
    :type valid: bool, optional
    """

    def __init__(
        self,
        amount_off: float = None,
        created: float = None,
        currency: str = None,
        description: str = None,
        discount_duration_in_uses: float = None,
        discount_valid_until: float = None,
        discount_validity_in_months: float = None,
        duration: Duration = None,
        duration_in_months: float = None,
        id_: str = None,
        max_redemptions: float = None,
        metadata: dict = None,
        percent_off: float = None,
        redeem_by: float = None,
        times_redeemed: float = None,
        valid: bool = None,
    ):
        """The coupon can be applied to the following objects - * Subscription - All charges in the subscription are subject to the discount described in the coupon. To add a coupon to a subscription, first use Create Coupon. Then add the coupon ID to the coupon field of the subscription with Create Subscription or Update Subscription. * Customer - The discount applies to all charges to the customer. To add a coupon to a customer, first use Create Coupon. Then add the coupon ID to the coupon field of the customer with Create Customer or Update Customer. * Order - The discount applies to a single order. To add a coupon to an order, first use Create Coupon. The duration field of the coupon must be set to repeating, and the duration_in_months and discount_duration_in_uses fields must be set to 1. Then add the coupon ID to the coupon field of the order with Create an Order or Update Order. A coupon can be applied to one or more customers or subscriptions. However, a subscription can have only one coupon, and the only way to assign two or more coupons to a customer is to assign them to separate subscriptions. If there is a coupon for the customer and another coupon for the customer's subscription, the subscription coupon takes precedence.

        :param amount_off: The amount of money to subtract from the payment. Decimal. Range: Positive decimal number, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015., defaults to None
        :type amount_off: float, optional
        :param created: When the coupon was created, in Unix time. Response only., defaults to None
        :type created: float, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param description: Description of the coupon., defaults to None
        :type description: str, optional
        :param discount_duration_in_uses: The number of times that the coupon can be redeemed by a specific customer or subscription. Relevant when duration is repeating and redeem_by is not set., defaults to None
        :type discount_duration_in_uses: float, optional
        :param discount_valid_until: The final date that a discount can be used, in Unix time. Relevant when discount_validity_in_months is not set., defaults to None
        :type discount_valid_until: float, optional
        :param discount_validity_in_months: The number of months after the discount is created that it can be used. Relevant when discount_valid_until is not set., defaults to None
        :type discount_validity_in_months: float, optional
        :param duration: Determines how long the discount remains in effect. One of the following:  * forever  * repeating , defaults to None
        :type duration: Duration, optional
        :param duration_in_months: Determines the number of months that the coupon remains in effect after its creation. Integer. Required when duration is repeating., defaults to None
        :type duration_in_months: float, optional
        :param id_: Unique ID for this coupon. English alphanumeric characters with no special characters except underscore. If the merchant does not define an ID, Rapyd generates a string starting with **coupon_**., defaults to None
        :type id_: str, optional
        :param max_redemptions: Determines the number of times the coupon can be redeemed. The number of customers, subscriptions or orders that the coupon is applied to cannot exceed this number. Integer., defaults to None
        :type max_redemptions: float, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param percent_off: Determines the size of the discount, measured in percent. Integer. Range: 1-100., defaults to None
        :type percent_off: float, optional
        :param redeem_by: The last time at which the coupon can be redeemed, in Unix time. After the redeem_by date, the coupon can no longer be applied to new customers. 'redeem_by' takes precedence over the setting of **discount_duration_in_uses** and **duration_in_months**., defaults to None
        :type redeem_by: float, optional
        :param times_redeemed: Indicates the number of times the coupon has been applied to a customer, subscription or invoice. If the discount has been used more than one time by a customer, subscription or invoice, the number of redemptions after the first is also added to this value. Integer. When this value equals **max_redemptions**, the coupon becomes no longer valid. Response only., defaults to None
        :type times_redeemed: float, optional
        :param valid: Indicates whether the coupon can be redeemed., defaults to None
        :type valid: bool, optional
        """
        self.amount_off = self._define_number("amount_off", amount_off, nullable=True)
        self.created = self._define_number("created", created, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.description = self._define_str("description", description, nullable=True)
        self.discount_duration_in_uses = self._define_number(
            "discount_duration_in_uses", discount_duration_in_uses, nullable=True
        )
        self.discount_valid_until = self._define_number(
            "discount_valid_until", discount_valid_until, nullable=True
        )
        self.discount_validity_in_months = self._define_number(
            "discount_validity_in_months", discount_validity_in_months, nullable=True
        )
        self.duration = (
            self._enum_matching(duration, Duration.list(), "duration")
            if duration
            else None
        )
        self.duration_in_months = self._define_number(
            "duration_in_months", duration_in_months, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.max_redemptions = self._define_number(
            "max_redemptions", max_redemptions, nullable=True
        )
        self.metadata = metadata
        self.percent_off = self._define_number(
            "percent_off", percent_off, nullable=True
        )
        self.redeem_by = self._define_number("redeem_by", redeem_by, nullable=True)
        self.times_redeemed = self._define_number(
            "times_redeemed", times_redeemed, nullable=True
        )
        self.valid = valid



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class DisputeStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACT: "ACT"
    :vartype ACT: str
    :cvar RVW: "RVW"
    :vartype RVW: str
    :cvar PRA: "PRA"
    :vartype PRA: str
    :cvar ARB: "ARB"
    :vartype ARB: str
    :cvar LOS: "LOS"
    :vartype LOS: str
    :cvar WIN: "WIN"
    :vartype WIN: str
    :cvar REV: "REV"
    :vartype REV: str
    """

    ACT = "ACT"
    RVW = "RVW"
    PRA = "PRA"
    ARB = "ARB"
    LOS = "LOS"
    WIN = "WIN"
    REV = "REV"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, DisputeStatus._member_map_.values()))


@JsonMap({"id_": "id"})
class Dispute(BaseModel):
    """Describes the fields contained in REST messages and webhooks for disputes of payments.

    :param id_: Reserved., defaults to None
    :type id_: str, optional
    :param token: ID of the dispute. String starting with **dispute_**., defaults to None
    :type token: str, optional
    :param status: Indicates the status of the dispute operation. One of the following values: * ACT (Active) - The dispute was initiated and is awaiting action by the merchant. * RVW (Review) - Rapyd is reviewing the merchant's evidence contesting the dispute. * PRA (Pre-Arbitration) - Rapyd is reviewing the merchant's evidence contesting a previously contested dispute. * ARB (Arbitration) - The merchant rejected a pre-arbitration outcome. Rapyd is awaiting a ruling by an arbitration committee from the card schemes. * LOS (Lose) - The merchant lost the dispute. Funds were deducted from the merchant wallet. Final status. * WIN (Win) - The merchant won the dispute. Funds were credited to the merchant wallet. Final status. * REV (Reverse) - The card issuer reversed the dispute. Funds were credited to the merchant wallet. Final status. , defaults to None
    :type status: DisputeStatus, optional
    :param amount: Amount that Rapyd deducted from the merchant due to the dispute., defaults to None
    :type amount: float, optional
    :param currency: Three-letter ISO 4217 code for the currency used in the amount field., defaults to None
    :type currency: str, optional
    :param dispute_category: The category of dispute that was provided by the card network., defaults to None
    :type dispute_category: str, optional
    :param dispute_reason_description: A short description of the reason for the dispute., defaults to None
    :type dispute_reason_description: str, optional
    :param original_transaction_currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type original_transaction_currency: str, optional
    :param original_transaction_amount: Amount of the original payment., defaults to None
    :type original_transaction_amount: float, optional
    :param original_dispute_amount: Amount of the dispute, as reported to Rapyd., defaults to None
    :type original_dispute_amount: float, optional
    :param original_dispute_currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type original_dispute_currency: str, optional
    :param original_transaction_id: ID of the payment that the dispute is charged against. String starting with **payment_**., defaults to None
    :type original_transaction_id: str, optional
    :param ewallet_id: ID of the wallet that the money is paid into. String starting with **ewallet_**. Relevant when the request includes a single wallet. Response only., defaults to None
    :type ewallet_id: str, optional
    :param central_processing_date: The date the transaction was processed by the card network., defaults to None
    :type central_processing_date: float, optional
    :param created_at: Time of creation of the dispute, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param updated_at: Time that the dispute was last updated, in Unix time. Response only., defaults to None
    :type updated_at: float, optional
    :param due_date: The latest date for the dispute to be contested in Unix time. Response only., defaults to None
    :type due_date: float, optional
    :param payment_method: ID of the payment method. String starting with **card_**., defaults to None
    :type payment_method: str, optional
    :param payment_method_data: Details of the payment method that was used for the payment. See Payment Method Data Object. Response only., defaults to None
    :type payment_method_data: dict, optional
    :param rate: FX rate. Relevant to FX payments., defaults to None
    :type rate: float, optional
    :param evidence: Reserved., defaults to None
    :type evidence: str, optional
    :param evidence_reason_code: Reserved., defaults to None
    :type evidence_reason_code: str, optional
    :param pre_dispute: When true, indicates that the disputed payment was automatically refunded based on pre-determined criteria. Response only., defaults to None
    :type pre_dispute: bool, optional
    :param arn: An Acquirer Reference Number (ARN) is a unique number assigned to a credit card transaction as it moves through the payment flow. Banks can use it to help trace the payment/ refund if it appears to be missing. An ARN will be available for Visa and Mastercard charges. , defaults to None
    :type arn: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        token: str = None,
        status: DisputeStatus = None,
        amount: float = None,
        currency: str = None,
        dispute_category: str = None,
        dispute_reason_description: str = None,
        original_transaction_currency: str = None,
        original_transaction_amount: float = None,
        original_dispute_amount: float = None,
        original_dispute_currency: str = None,
        original_transaction_id: str = None,
        ewallet_id: str = None,
        central_processing_date: float = None,
        created_at: float = None,
        updated_at: float = None,
        due_date: float = None,
        payment_method: str = None,
        payment_method_data: dict = None,
        rate: float = None,
        evidence: str = None,
        evidence_reason_code: str = None,
        pre_dispute: bool = None,
        arn: str = None,
    ):
        """Describes the fields contained in REST messages and webhooks for disputes of payments.

        :param id_: Reserved., defaults to None
        :type id_: str, optional
        :param token: ID of the dispute. String starting with **dispute_**., defaults to None
        :type token: str, optional
        :param status: Indicates the status of the dispute operation. One of the following values: * ACT (Active) - The dispute was initiated and is awaiting action by the merchant. * RVW (Review) - Rapyd is reviewing the merchant's evidence contesting the dispute. * PRA (Pre-Arbitration) - Rapyd is reviewing the merchant's evidence contesting a previously contested dispute. * ARB (Arbitration) - The merchant rejected a pre-arbitration outcome. Rapyd is awaiting a ruling by an arbitration committee from the card schemes. * LOS (Lose) - The merchant lost the dispute. Funds were deducted from the merchant wallet. Final status. * WIN (Win) - The merchant won the dispute. Funds were credited to the merchant wallet. Final status. * REV (Reverse) - The card issuer reversed the dispute. Funds were credited to the merchant wallet. Final status. , defaults to None
        :type status: DisputeStatus, optional
        :param amount: Amount that Rapyd deducted from the merchant due to the dispute., defaults to None
        :type amount: float, optional
        :param currency: Three-letter ISO 4217 code for the currency used in the amount field., defaults to None
        :type currency: str, optional
        :param dispute_category: The category of dispute that was provided by the card network., defaults to None
        :type dispute_category: str, optional
        :param dispute_reason_description: A short description of the reason for the dispute., defaults to None
        :type dispute_reason_description: str, optional
        :param original_transaction_currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type original_transaction_currency: str, optional
        :param original_transaction_amount: Amount of the original payment., defaults to None
        :type original_transaction_amount: float, optional
        :param original_dispute_amount: Amount of the dispute, as reported to Rapyd., defaults to None
        :type original_dispute_amount: float, optional
        :param original_dispute_currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type original_dispute_currency: str, optional
        :param original_transaction_id: ID of the payment that the dispute is charged against. String starting with **payment_**., defaults to None
        :type original_transaction_id: str, optional
        :param ewallet_id: ID of the wallet that the money is paid into. String starting with **ewallet_**. Relevant when the request includes a single wallet. Response only., defaults to None
        :type ewallet_id: str, optional
        :param central_processing_date: The date the transaction was processed by the card network., defaults to None
        :type central_processing_date: float, optional
        :param created_at: Time of creation of the dispute, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param updated_at: Time that the dispute was last updated, in Unix time. Response only., defaults to None
        :type updated_at: float, optional
        :param due_date: The latest date for the dispute to be contested in Unix time. Response only., defaults to None
        :type due_date: float, optional
        :param payment_method: ID of the payment method. String starting with **card_**., defaults to None
        :type payment_method: str, optional
        :param payment_method_data: Details of the payment method that was used for the payment. See Payment Method Data Object. Response only., defaults to None
        :type payment_method_data: dict, optional
        :param rate: FX rate. Relevant to FX payments., defaults to None
        :type rate: float, optional
        :param evidence: Reserved., defaults to None
        :type evidence: str, optional
        :param evidence_reason_code: Reserved., defaults to None
        :type evidence_reason_code: str, optional
        :param pre_dispute: When true, indicates that the disputed payment was automatically refunded based on pre-determined criteria. Response only., defaults to None
        :type pre_dispute: bool, optional
        :param arn: An Acquirer Reference Number (ARN) is a unique number assigned to a credit card transaction as it moves through the payment flow. Banks can use it to help trace the payment/ refund if it appears to be missing. An ARN will be available for Visa and Mastercard charges. , defaults to None
        :type arn: str, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.token = self._define_str("token", token, nullable=True)
        self.status = (
            self._enum_matching(status, DisputeStatus.list(), "status")
            if status
            else None
        )
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.dispute_category = self._define_str(
            "dispute_category", dispute_category, nullable=True
        )
        self.dispute_reason_description = self._define_str(
            "dispute_reason_description", dispute_reason_description, nullable=True
        )
        self.original_transaction_currency = self._define_str(
            "original_transaction_currency",
            original_transaction_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.original_transaction_amount = self._define_number(
            "original_transaction_amount", original_transaction_amount, nullable=True
        )
        self.original_dispute_amount = self._define_number(
            "original_dispute_amount", original_dispute_amount, nullable=True
        )
        self.original_dispute_currency = self._define_str(
            "original_dispute_currency",
            original_dispute_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.original_transaction_id = self._define_str(
            "original_transaction_id", original_transaction_id, nullable=True
        )
        self.ewallet_id = self._define_str("ewallet_id", ewallet_id, nullable=True)
        self.central_processing_date = self._define_number(
            "central_processing_date", central_processing_date, nullable=True
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.updated_at = self._define_number("updated_at", updated_at, nullable=True)
        self.due_date = self._define_number("due_date", due_date, nullable=True)
        self.payment_method = self._define_str(
            "payment_method", payment_method, nullable=True
        )
        self.payment_method_data = payment_method_data
        self.rate = self._define_number("rate", rate, nullable=True)
        self.evidence = self._define_str("evidence", evidence, nullable=True)
        self.evidence_reason_code = self._define_str(
            "evidence_reason_code", evidence_reason_code, nullable=True
        )
        self.pre_dispute = pre_dispute
        self.arn = self._define_str("arn", arn, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class IssuingActivateCardBody(BaseModel):
    """IssuingActivateCardBody

    :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param ewallet_contact: ID of the wallet contact that the card is assigned to. Must have a valid phone number. String starting with **cont_**.
    :type ewallet_contact: str
    :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'.\<BR\>* When this parameter is null, the language of the user's browser is used.\<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation.\<BR\>**Range**: 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param personalize: When **true**, connects an issued card to a wallet contact. Relevant to a card that was issued in bulk and is not assigned to a specific person., defaults to None
    :type personalize: bool, optional
    :param skip_pin: When **true**, the customer cannot reset the PIN., defaults to None
    :type skip_pin: bool, optional
    """

    def __init__(
        self,
        ewallet_contact: str,
        cancel_url: str = None,
        complete_url: str = None,
        language: str = None,
        page_expiration: float = None,
        personalize: bool = None,
        skip_pin: bool = None,
    ):
        """IssuingActivateCardBody

        :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param ewallet_contact: ID of the wallet contact that the card is assigned to. Must have a valid phone number. String starting with **cont_**.
        :type ewallet_contact: str
        :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'.\<BR\>* When this parameter is null, the language of the user's browser is used.\<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation.\<BR\>**Range**: 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param personalize: When **true**, connects an issued card to a wallet contact. Relevant to a card that was issued in bulk and is not assigned to a specific person., defaults to None
        :type personalize: bool, optional
        :param skip_pin: When **true**, the customer cannot reset the PIN., defaults to None
        :type skip_pin: bool, optional
        """
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.ewallet_contact = ewallet_contact
        self.language = self._define_str("language", language, nullable=True)
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.personalize = personalize
        self.skip_pin = skip_pin



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1cnltermination_query_queried_merchant_principals_address import (
    V1cnlterminationQueryQueriedMerchantPrincipalsAddress,
)


@JsonMap({})
class V1cnlterminationQueryQueriedMerchantPrincipals(BaseModel):
    """Details of the registered principal owners of the merchant.<BR> Maximum - 3.

    :param first_name: The first name of the owner.\<BR\>String that starts and ends with any combination of characters, whitespace, a specified range of Latin letters with diacritics, and a specific set of special characters.\<BR\>Length: 1-35, defaults to None
    :type first_name: str, optional
    :param middle_initial: The initial letter of the owner's middle name.\<BR\>String that starts and ends with any combination of characters, whitespace, a specified range of Latin letters with diacritics, and a specific set of special characters.\<BR\>Length: Maximum - 1, defaults to None
    :type middle_initial: str, optional
    :param last_name: The family name of the owner.\<BR\>String that starts and ends with any combination of characters, whitespace, a specified range of Latin letters with diacritics, and a specific set of special characters.\<BR\>Length: 1-40, defaults to None
    :type last_name: str, optional
    :param email: The primary email address of the merchant.\<BR\>String that starts with a combination of letters, numbers, ., _, %, or - before an **@** sign. After the **@** sign, a domain name with letters, numbers, ., or -, ending with a dot and a 2 to 4 letter domain type, such as **.com**. For example: **billsmith@snapphoto.com**\<BR\>Length: Maximum - 90, defaults to None
    :type email: str, optional
    :param phone_number: The phone number of the owner.\<BR\> Phone number in the format +[country code]/[phone number], where the forward slash (/) represents a separator between the country code and the telephone number.\<BR\>The plus sign before the country code is optional. The country code may only contain digits.\<BR\>The slash is required and must follow the country code.\<BR\>The telephone number may include blank spaces and hyphens (-).\<BR\>For example, in **+1/555 555-5555**, the country code is 1 and the telephone number is 5555555555.\<BR\>Length: 8-15, defaults to None
    :type phone_number: str, optional
    :param address: Details of the owner's business address., defaults to None
    :type address: V1cnlterminationQueryQueriedMerchantPrincipalsAddress, optional
    """

    def __init__(
        self,
        first_name: str = None,
        middle_initial: str = None,
        last_name: str = None,
        email: str = None,
        phone_number: str = None,
        address: V1cnlterminationQueryQueriedMerchantPrincipalsAddress = None,
    ):
        """Details of the registered principal owners of the merchant.<BR> Maximum - 3.

        :param first_name: The first name of the owner.\<BR\>String that starts and ends with any combination of characters, whitespace, a specified range of Latin letters with diacritics, and a specific set of special characters.\<BR\>Length: 1-35, defaults to None
        :type first_name: str, optional
        :param middle_initial: The initial letter of the owner's middle name.\<BR\>String that starts and ends with any combination of characters, whitespace, a specified range of Latin letters with diacritics, and a specific set of special characters.\<BR\>Length: Maximum - 1, defaults to None
        :type middle_initial: str, optional
        :param last_name: The family name of the owner.\<BR\>String that starts and ends with any combination of characters, whitespace, a specified range of Latin letters with diacritics, and a specific set of special characters.\<BR\>Length: 1-40, defaults to None
        :type last_name: str, optional
        :param email: The primary email address of the merchant.\<BR\>String that starts with a combination of letters, numbers, ., _, %, or - before an **@** sign. After the **@** sign, a domain name with letters, numbers, ., or -, ending with a dot and a 2 to 4 letter domain type, such as **.com**. For example: **billsmith@snapphoto.com**\<BR\>Length: Maximum - 90, defaults to None
        :type email: str, optional
        :param phone_number: The phone number of the owner.\<BR\> Phone number in the format +[country code]/[phone number], where the forward slash (/) represents a separator between the country code and the telephone number.\<BR\>The plus sign before the country code is optional. The country code may only contain digits.\<BR\>The slash is required and must follow the country code.\<BR\>The telephone number may include blank spaces and hyphens (-).\<BR\>For example, in **+1/555 555-5555**, the country code is 1 and the telephone number is 5555555555.\<BR\>Length: 8-15, defaults to None
        :type phone_number: str, optional
        :param address: Details of the owner's business address., defaults to None
        :type address: V1cnlterminationQueryQueriedMerchantPrincipalsAddress, optional
        """
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.middle_initial = self._define_str(
            "middle_initial", middle_initial, nullable=True
        )
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.email = self._define_str("email", email, nullable=True)
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.address = self._define_object(
            address, V1cnlterminationQueryQueriedMerchantPrincipalsAddress
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id", "type_": "type"})
class MassPayoutResponse(BaseModel):
    """MassPayoutResponse

    :param created_at: Time of creation of the batch operation, in Unix time (seconds)., defaults to None
    :type created_at: float, optional
    :param id_: ID of the batch operation. String starting with **batch_**.Three-letter ISO 4217 code for the currency of an existing account., defaults to None
    :type id_: str, optional
    :param original_name: Filename of the uploaded batch file., defaults to None
    :type original_name: str, optional
    :param status: Status of the batch operation - "NEW"., defaults to None
    :type status: str, optional
    :param type_: Batch operation type - "mass_payout_pci"., defaults to None
    :type type_: str, optional
    """

    def __init__(
        self,
        created_at: float = None,
        id_: str = None,
        original_name: str = None,
        status: str = None,
        type_: str = None,
    ):
        """MassPayoutResponse

        :param created_at: Time of creation of the batch operation, in Unix time (seconds)., defaults to None
        :type created_at: float, optional
        :param id_: ID of the batch operation. String starting with **batch_**.Three-letter ISO 4217 code for the currency of an existing account., defaults to None
        :type id_: str, optional
        :param original_name: Filename of the uploaded batch file., defaults to None
        :type original_name: str, optional
        :param status: Status of the batch operation - "NEW"., defaults to None
        :type status: str, optional
        :param type_: Batch operation type - "mass_payout_pci"., defaults to None
        :type type_: str, optional
        """
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.original_name = self._define_str(
            "original_name", original_name, nullable=True
        )
        self.status = self._define_str("status", status, nullable=True)
        self.type_ = self._define_str("type_", type_, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .entity_type_verify import EntityTypeVerify


@JsonMap({})
class InlineResponse200_113Data(BaseModel):
    """InlineResponse200_113Data

    :param application_type: Code for the type of application. String starting with typ_., defaults to None
    :type application_type: str, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param entity_type: entity_type, defaults to None
    :type entity_type: EntityTypeVerify, optional
    """

    def __init__(
        self,
        application_type: str = None,
        country: str = None,
        entity_type: EntityTypeVerify = None,
    ):
        """InlineResponse200_113Data

        :param application_type: Code for the type of application. String starting with typ_., defaults to None
        :type application_type: str, optional
        :param country: country, defaults to None
        :type country: str, optional
        :param entity_type: entity_type, defaults to None
        :type entity_type: EntityTypeVerify, optional
        """
        self.application_type = self._define_str(
            "application_type", application_type, nullable=True
        )
        self.country = self._define_str(
            "country",
            country,
            nullable=True,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.entity_type = (
            self._enum_matching(entity_type, EntityTypeVerify.list(), "entity_type")
            if entity_type
            else None
        )



import hashlib
import base64
import os
import urllib.parse
from datetime import datetime
import string
from random import *
import hmac
import json
import time
from decimal import Decimal


def remove_trailing_zeros(value):
    if isinstance(value, float):
        return str(Decimal(value).normalize())
    elif isinstance(value, dict):
        return {k: remove_trailing_zeros(v) for k, v in value.items()}
    elif isinstance(value, list):
        return [remove_trailing_zeros(v) for v in value]
    else:
        return value


# salt: randomly generated for each request.
def generate_salt(length=12):
    return "".join(sample(string.ascii_letters + string.digits, length))


# Current Unix time (seconds).
def get_unix_time(days=0, hours=0, minutes=0, seconds=0):
    return int(time.time())


def update_timestamp_salt_sig(access_key, secret_key, http_method, path, body):
    if path.startswith("http"):
        path = path[path.find(f"/v1") :]

    salt = generate_salt()
    timestamp = get_unix_time()
    to_sign = (http_method, path, salt, str(timestamp), access_key, secret_key, body)

    h = hmac.new(
        secret_key.encode("utf-8"), "".join(to_sign).encode("utf-8"), hashlib.sha256
    )
    signature = base64.urlsafe_b64encode(str.encode(h.hexdigest()))
    return salt, timestamp, signature


def current_sig_headers(access_key, salt, timestamp, signature):
    sig_headers = {
        "access_key": access_key,
        "salt": salt,
        "timestamp": str(timestamp),
        "signature": signature,
        "idempotency": str(get_unix_time()) + salt,
    }
    return sig_headers


# http_method = get|put|post|delete - must be lowercase
# path = Portion after the base URL.
# body = JSON body with no whitespace except in strings.
def pre_call(access_key, secret_key, http_method, path, body=None):
    str_body = (
        json.dumps(body, separators=(",", ":"), ensure_ascii=False) if body else ""
    )
    salt, timestamp, signature = update_timestamp_salt_sig(
        access_key=access_key,
        secret_key=secret_key,
        http_method=http_method,
        path=path,
        body=str_body,
    )
    return str_body.encode("utf-8"), salt, timestamp, signature


def create_headers(access_key, secret_key, http_method, url, body=None):
    body, salt, timestamp, signature = pre_call(
        access_key=access_key,
        secret_key=secret_key,
        http_method=http_method,
        path=url,
        body=body,
    )
    return body, current_sig_headers(access_key, salt, timestamp, signature)


def get_keys(base_url):
    access_key = os.environ["ACCESS_KEY"]
    secret_key = os.environ["SECRET_KEY"]
    return access_key, secret_key


class Request:
    def __init__(self, method, url, headers, body=""):
        self.method = method
        self.url = url
        self.headers = headers
        self.body = body

    def __str__(self):
        return f"method={self.method}, url={self.url}, headers={self.headers}, body={self.body})"


class Response:
    def __init__(self, status, headers, body):
        self.status = status
        self.headers = headers
        self.body = body

    def __str__(self):
        return "Response(status={}, headers={}, body={})".format(
            self.status, self.headers, self.body
        )


class CustomHook:
    def before_request(self, request: Request):
        parsed = urllib.parse.urlparse(request.url)

        http_method = (
            f"{request.method}".lower()
        )  # get|put|post|delete - must be lowercase
        base_url = (
            f"{parsed.scheme}://{parsed.hostname}"  # base url without query string
        )
        request.body = remove_trailing_zeros(request.body)
        access_key, secret_key = get_keys(base_url)
        body, headers = create_headers(
            access_key, secret_key, http_method, request.url, request.body
        )

        request.headers = dict(request.headers, **headers)

    def after_response(self, request: Request, response: Response):
        print("after_response")

    def on_error(self, error: Exception, request: Request, response: Response):
        print("on_error")



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardsSimulateBlockBody(BaseModel):
    """CardsSimulateBlockBody

    :param blocked_reason: Reason for blocking the card., defaults to None
    :type blocked_reason: str, optional
    :param card_id: ID of the card. String starting with **card_**.
    :type card_id: str
    """

    def __init__(self, card_id: str, blocked_reason: str = None):
        """CardsSimulateBlockBody

        :param blocked_reason: Reason for blocking the card., defaults to None
        :type blocked_reason: str, optional
        :param card_id: ID of the card. String starting with **card_**.
        :type card_id: str
        """
        self.blocked_reason = self._define_str(
            "blocked_reason", blocked_reason, nullable=True
        )
        self.card_id = card_id



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class SubscriptionHostedPageReponseCustomElements(BaseModel):
    """Description of the payment transaction.

    :param billing_address_collect: Determines whether the customer is asked to fill in the billing address. Relevant when a card payment method is selected. \<BR\> * **true** - The address fields appear on the checkout page.\<BR\> * **false** - The address fields appear only if the country is **US, **GB** or **CA**., defaults to None
    :type billing_address_collect: bool, optional
    :param cardholder_name: The name of the card owner, printed on the front of the card., defaults to None
    :type cardholder_name: str, optional
    :param display_description: Determines whether the checkout page displays the payment description.\<BR\>* **true** - The payment description appears.\<BR\> * **false** - The payment description does not appear.\<BR\> Relevant when description is passed in the Create Checkout Page request., defaults to None
    :type display_description: bool, optional
    :param dynamic_currency_conversion: Determines whether the checkout page displays multiple currency options for a payment.\<BR\> * **true** - Multiple currency options appear.\<BR\> * **false** - Currency options do not appear.\<BR\>When the customer selects the requested currency, the checkout page displays the following information:\<BR\>* The original amount and currency.\<BR\> * The converted amount in the requested currency.\<BR\> * The exchange rate.\<BR\> Relevant when: \<BR\> * The Create Checkout Page request passes requested_currency.\<BR\> * `fixed_side` is **buy**. \<BR\> * One or more payment methods support the values for `currency` and `requested_currency`., defaults to None
    :type dynamic_currency_conversion: bool, optional
    :param merchant_color: reserved, defaults to None
    :type merchant_color: str, optional
    :param merchant_currency_only: In a payment with FX where fixed_side=**buy**, determines whether the buyer's currency and the exchange rate appear. One of the following:\<BR\> * **true** - The currency and the exchange rate are hidden.\<BR\> * **false** - The currency and the exchange rate appear., defaults to None
    :type merchant_currency_only: bool, optional
    :param payment_fees_display: Determines whether payment fees appear on the checkout page.\<BR\> * **true** - Payment fees appear when the `payment_fees` object is set in the 'Create Checkout Page' request.\<BR\> * **false** - Payment fees do not appear., defaults to None
    :type payment_fees_display: bool, optional
    :param required_customer_fields: Indicates the list of fields that the customer has to fill in on the payment page before completing the payment via hosted checkout. Valid values include:\<BR\>* **name** – The customer's full name (default).\<BR\> * **email** – The customer's email address.\<BR\> * **phone_number** – The customer's phone number.\<BR\> * **address** – The address of the customer., defaults to None
    :type required_customer_fields: any, optional
    :param save_card_default: Determines whether the save card checkbox is checked by default.\<BR\> * **true** - The **save card** checkbox is checked.\<BR\> * **false** - The **save card** checkbox is cleared.\<BR\> Relevant when `customer_id` is passed in the 'Create Checkout Page' request., defaults to None
    :type save_card_default: bool, optional
    """

    def __init__(
        self,
        billing_address_collect: bool = None,
        cardholder_name: str = None,
        display_description: bool = None,
        dynamic_currency_conversion: bool = None,
        merchant_color: str = None,
        merchant_currency_only: bool = None,
        payment_fees_display: bool = None,
        required_customer_fields: any = None,
        save_card_default: bool = None,
    ):
        """Description of the payment transaction.

        :param billing_address_collect: Determines whether the customer is asked to fill in the billing address. Relevant when a card payment method is selected. \<BR\> * **true** - The address fields appear on the checkout page.\<BR\> * **false** - The address fields appear only if the country is **US, **GB** or **CA**., defaults to None
        :type billing_address_collect: bool, optional
        :param cardholder_name: The name of the card owner, printed on the front of the card., defaults to None
        :type cardholder_name: str, optional
        :param display_description: Determines whether the checkout page displays the payment description.\<BR\>* **true** - The payment description appears.\<BR\> * **false** - The payment description does not appear.\<BR\> Relevant when description is passed in the Create Checkout Page request., defaults to None
        :type display_description: bool, optional
        :param dynamic_currency_conversion: Determines whether the checkout page displays multiple currency options for a payment.\<BR\> * **true** - Multiple currency options appear.\<BR\> * **false** - Currency options do not appear.\<BR\>When the customer selects the requested currency, the checkout page displays the following information:\<BR\>* The original amount and currency.\<BR\> * The converted amount in the requested currency.\<BR\> * The exchange rate.\<BR\> Relevant when: \<BR\> * The Create Checkout Page request passes requested_currency.\<BR\> * `fixed_side` is **buy**. \<BR\> * One or more payment methods support the values for `currency` and `requested_currency`., defaults to None
        :type dynamic_currency_conversion: bool, optional
        :param merchant_color: reserved, defaults to None
        :type merchant_color: str, optional
        :param merchant_currency_only: In a payment with FX where fixed_side=**buy**, determines whether the buyer's currency and the exchange rate appear. One of the following:\<BR\> * **true** - The currency and the exchange rate are hidden.\<BR\> * **false** - The currency and the exchange rate appear., defaults to None
        :type merchant_currency_only: bool, optional
        :param payment_fees_display: Determines whether payment fees appear on the checkout page.\<BR\> * **true** - Payment fees appear when the `payment_fees` object is set in the 'Create Checkout Page' request.\<BR\> * **false** - Payment fees do not appear., defaults to None
        :type payment_fees_display: bool, optional
        :param required_customer_fields: Indicates the list of fields that the customer has to fill in on the payment page before completing the payment via hosted checkout. Valid values include:\<BR\>* **name** – The customer's full name (default).\<BR\> * **email** – The customer's email address.\<BR\> * **phone_number** – The customer's phone number.\<BR\> * **address** – The address of the customer., defaults to None
        :type required_customer_fields: any, optional
        :param save_card_default: Determines whether the save card checkbox is checked by default.\<BR\> * **true** - The **save card** checkbox is checked.\<BR\> * **false** - The **save card** checkbox is cleared.\<BR\> Relevant when `customer_id` is passed in the 'Create Checkout Page' request., defaults to None
        :type save_card_default: bool, optional
        """
        self.billing_address_collect = billing_address_collect
        self.cardholder_name = self._define_str(
            "cardholder_name", cardholder_name, nullable=True
        )
        self.display_description = display_description
        self.dynamic_currency_conversion = dynamic_currency_conversion
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_currency_only = merchant_currency_only
        self.payment_fees_display = payment_fees_display
        self.required_customer_fields = required_customer_fields
        self.save_card_default = save_card_default



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .address_1 import Address1
from .discount import Discount
from .customer_payment_methods import CustomerPaymentMethods
from .subscription import Subscription


@JsonMap({"id_": "id"})
class Customer(BaseModel):
    """customer

    :param addresses: addresses, defaults to None
    :type addresses: List[Address1], optional
    :param business_vat_id: The tax ID number of the customer. Relevant when the customer is a business, defaults to None
    :type business_vat_id: str, optional
    :param created_at: Time of creation of this customer, in Unix time. Response only, defaults to None
    :type created_at: int, optional
    :param default_payment_method: The payment method that is used when the 'payment' object or subscription does not specify a payment method. The value must also appear in the payment_methods list. The payment method is referenced by its name field., defaults to None
    :type default_payment_method: str, optional
    :param delinquent: Indicates whether there is currently a failure of an automatic payment that is part of a subscription, or an invoice that was not paid when due. Response only. true - The account is delinquent. false - The account is current, defaults to None
    :type delinquent: bool, optional
    :param description: A text description of the customer, defaults to None
    :type description: str, optional
    :param discount: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
    :type discount: Discount, optional
    :param email: Customer's email address. Maximum 512 characters, defaults to None
    :type email: str, optional
    :param ewallet: ID of the wallet that is linked to the customer. String starting with **ewallet_**. Each wallet can be associated with only one customer, defaults to None
    :type ewallet: str, optional
    :param id_: ID of the Customer object. String starting with **cus_**, defaults to None
    :type id_: str, optional
    :param invoice_prefix: A custom string that is prefixed to all invoices for this customer. For more information see Invoice Object at "https://docs.rapyd.net/en/invoice-364856.html", defaults to None
    :type invoice_prefix: str, optional
    :param metadata: A JSON object defined by the client, defaults to None
    :type metadata: dict, optional
    :param name: The name of the customer
    :type name: str
    :param payment_methods: An object containing the following fields - * data - A list of up to three payment methods. For more information, see Customer Payment Method Object. * has_more - Indicates whether there are more than three payment methods for this customer. * total_count - Total number of payment methods for this customer. * url - URL for requesting all of the payment methods for this customer.., defaults to None
    :type payment_methods: CustomerPaymentMethods, optional
    :param phone_number: Customer's primary phone number in E.164 format. The merchant is responsible for verifying that the number is correct. One method of verifying could be to send an activation code to the phone by SMS, with a limited time for response., defaults to None
    :type phone_number: str, optional
    :param subscriptions: subscriptions, defaults to None
    :type subscriptions: List[Subscription], optional
    """

    def __init__(
        self,
        name: str,
        addresses: List[Address1] = None,
        business_vat_id: str = None,
        created_at: int = None,
        default_payment_method: str = None,
        delinquent: bool = None,
        description: str = None,
        discount: Discount = None,
        email: str = None,
        ewallet: str = None,
        id_: str = None,
        invoice_prefix: str = None,
        metadata: dict = None,
        payment_methods: CustomerPaymentMethods = None,
        phone_number: str = None,
        subscriptions: List[Subscription] = None,
    ):
        """customer

        :param addresses: addresses, defaults to None
        :type addresses: List[Address1], optional
        :param business_vat_id: The tax ID number of the customer. Relevant when the customer is a business, defaults to None
        :type business_vat_id: str, optional
        :param created_at: Time of creation of this customer, in Unix time. Response only, defaults to None
        :type created_at: int, optional
        :param default_payment_method: The payment method that is used when the 'payment' object or subscription does not specify a payment method. The value must also appear in the payment_methods list. The payment method is referenced by its name field., defaults to None
        :type default_payment_method: str, optional
        :param delinquent: Indicates whether there is currently a failure of an automatic payment that is part of a subscription, or an invoice that was not paid when due. Response only. true - The account is delinquent. false - The account is current, defaults to None
        :type delinquent: bool, optional
        :param description: A text description of the customer, defaults to None
        :type description: str, optional
        :param discount: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
        :type discount: Discount, optional
        :param email: Customer's email address. Maximum 512 characters, defaults to None
        :type email: str, optional
        :param ewallet: ID of the wallet that is linked to the customer. String starting with **ewallet_**. Each wallet can be associated with only one customer, defaults to None
        :type ewallet: str, optional
        :param id_: ID of the Customer object. String starting with **cus_**, defaults to None
        :type id_: str, optional
        :param invoice_prefix: A custom string that is prefixed to all invoices for this customer. For more information see Invoice Object at "https://docs.rapyd.net/en/invoice-364856.html", defaults to None
        :type invoice_prefix: str, optional
        :param metadata: A JSON object defined by the client, defaults to None
        :type metadata: dict, optional
        :param name: The name of the customer
        :type name: str
        :param payment_methods: An object containing the following fields - * data - A list of up to three payment methods. For more information, see Customer Payment Method Object. * has_more - Indicates whether there are more than three payment methods for this customer. * total_count - Total number of payment methods for this customer. * url - URL for requesting all of the payment methods for this customer.., defaults to None
        :type payment_methods: CustomerPaymentMethods, optional
        :param phone_number: Customer's primary phone number in E.164 format. The merchant is responsible for verifying that the number is correct. One method of verifying could be to send an activation code to the phone by SMS, with a limited time for response., defaults to None
        :type phone_number: str, optional
        :param subscriptions: subscriptions, defaults to None
        :type subscriptions: List[Subscription], optional
        """
        self.addresses = self._define_list(addresses, Address1)
        self.business_vat_id = self._define_str(
            "business_vat_id", business_vat_id, nullable=True
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.default_payment_method = self._define_str(
            "default_payment_method", default_payment_method, nullable=True
        )
        self.delinquent = delinquent
        self.description = self._define_str("description", description, nullable=True)
        self.discount = self._define_object(discount, Discount)
        self.email = self._define_str("email", email, nullable=True)
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.invoice_prefix = self._define_str(
            "invoice_prefix", invoice_prefix, nullable=True
        )
        self.metadata = metadata
        self.name = name
        self.payment_methods = self._define_object(
            payment_methods, CustomerPaymentMethods
        )
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.subscriptions = self._define_list(subscriptions, Subscription)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class InlineResponse200_18DataTransactions(BaseModel):
    """InlineResponse200_18DataTransactions

    :param id_: ID of the issued card transaction object. String starting with cit_., defaults to None
    :type id_: str, optional
    :param ewallet: ID of the wallet the bank account number is issued to. String starting with ewallet_., defaults to None
    :type ewallet: str, optional
    :param account_id: ID of the virtual bank account., defaults to None
    :type account_id: str, optional
    :param account_id_type: Type of the virtual account number., defaults to None
    :type account_id_type: str, optional
    :param amount: Amount of the transaction., defaults to None
    :type amount: float, optional
    :param currency: Currency of the transaction. Three-letter ISO 4217., defaults to None
    :type currency: str, optional
    :param created_at: Time of creation of the transaction, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param original_amount: This field will show the initial transaction amount before conversion., defaults to None
    :type original_amount: float, optional
    :param original_currency: original_currency, defaults to None
    :type original_currency: str, optional
    :param fx_rate: This field will indicate the exchange rate applied for the currency conversion, defaults to None
    :type fx_rate: float, optional
    """

    def __init__(
        self,
        id_: str = None,
        ewallet: str = None,
        account_id: str = None,
        account_id_type: str = None,
        amount: float = None,
        currency: str = None,
        created_at: float = None,
        original_amount: float = None,
        original_currency: str = None,
        fx_rate: float = None,
    ):
        """InlineResponse200_18DataTransactions

        :param id_: ID of the issued card transaction object. String starting with cit_., defaults to None
        :type id_: str, optional
        :param ewallet: ID of the wallet the bank account number is issued to. String starting with ewallet_., defaults to None
        :type ewallet: str, optional
        :param account_id: ID of the virtual bank account., defaults to None
        :type account_id: str, optional
        :param account_id_type: Type of the virtual account number., defaults to None
        :type account_id_type: str, optional
        :param amount: Amount of the transaction., defaults to None
        :type amount: float, optional
        :param currency: Currency of the transaction. Three-letter ISO 4217., defaults to None
        :type currency: str, optional
        :param created_at: Time of creation of the transaction, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param original_amount: This field will show the initial transaction amount before conversion., defaults to None
        :type original_amount: float, optional
        :param original_currency: original_currency, defaults to None
        :type original_currency: str, optional
        :param fx_rate: This field will indicate the exchange rate applied for the currency conversion, defaults to None
        :type fx_rate: float, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.account_id = self._define_str("account_id", account_id, nullable=True)
        self.account_id_type = self._define_str(
            "account_id_type", account_id_type, nullable=True
        )
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.original_amount = self._define_number(
            "original_amount", original_amount, nullable=True
        )
        self.original_currency = self._define_str(
            "original_currency",
            original_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.fx_rate = self._define_number("fx_rate", fx_rate, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .address_1 import Address1
from .customer_request_payment_method import CustomerRequestPaymentMethod


@JsonMap({})
class V1CustomersBody(BaseModel):
    """V1CustomersBody

    :param addresses: Array of address objects associated with this customer. For more information, defaults to None
    :type addresses: List[Address1], optional
    :param business_vat_id: The tax ID number of the customer, defaults to None
    :type business_vat_id: str, optional
    :param coupon: The ID of a coupon that is assigned to this customer, defaults to None
    :type coupon: str, optional
    :param payment_method: The payment method that is used when the transaction does not specify a payment method. String starting with **card_** or other_., defaults to None
    :type payment_method: CustomerRequestPaymentMethod, optional
    :param description: A text description of the customer, defaults to None
    :type description: str, optional
    :param email: Customer's email address, defaults to None
    :type email: str, optional
    :param ewallet: ID of the wallet that is linked to the customer. String starting with **ewallet_**., defaults to None
    :type ewallet: str, optional
    :param invoice_prefix: A custom string that is prefixed to all invoices for this customer., defaults to None
    :type invoice_prefix: str, optional
    :param metadata: A JSON object defined by the Rapyd partner, defaults to None
    :type metadata: dict, optional
    :param name: The name of the individual customer or the business name
    :type name: str
    :param phone_number: Customer's primary phone number in E.164 format, defaults to None
    :type phone_number: str, optional
    """

    def __init__(
        self,
        name: str,
        addresses: List[Address1] = None,
        business_vat_id: str = None,
        coupon: str = None,
        payment_method: CustomerRequestPaymentMethod = None,
        description: str = None,
        email: str = None,
        ewallet: str = None,
        invoice_prefix: str = None,
        metadata: dict = None,
        phone_number: str = None,
    ):
        """V1CustomersBody

        :param addresses: Array of address objects associated with this customer. For more information, defaults to None
        :type addresses: List[Address1], optional
        :param business_vat_id: The tax ID number of the customer, defaults to None
        :type business_vat_id: str, optional
        :param coupon: The ID of a coupon that is assigned to this customer, defaults to None
        :type coupon: str, optional
        :param payment_method: The payment method that is used when the transaction does not specify a payment method. String starting with **card_** or other_., defaults to None
        :type payment_method: CustomerRequestPaymentMethod, optional
        :param description: A text description of the customer, defaults to None
        :type description: str, optional
        :param email: Customer's email address, defaults to None
        :type email: str, optional
        :param ewallet: ID of the wallet that is linked to the customer. String starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        :param invoice_prefix: A custom string that is prefixed to all invoices for this customer., defaults to None
        :type invoice_prefix: str, optional
        :param metadata: A JSON object defined by the Rapyd partner, defaults to None
        :type metadata: dict, optional
        :param name: The name of the individual customer or the business name
        :type name: str
        :param phone_number: Customer's primary phone number in E.164 format, defaults to None
        :type phone_number: str, optional
        """
        self.addresses = self._define_list(addresses, Address1)
        self.business_vat_id = self._define_str(
            "business_vat_id", business_vat_id, nullable=True
        )
        self.coupon = self._define_str("coupon", coupon, nullable=True)
        self.payment_method = self._define_object(
            payment_method, CustomerRequestPaymentMethod
        )
        self.description = self._define_str("description", description, nullable=True)
        self.email = self._define_str("email", email, nullable=True)
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.invoice_prefix = self._define_str(
            "invoice_prefix", invoice_prefix, nullable=True
        )
        self.metadata = metadata
        self.name = name
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
